# -*- coding: utf-8 -*-
from init import init
from interfaces.boot import main
from tools.logging import logging, LoggingToFile

if __name__ == '__main__':
    logging.set_logging_stream(LoggingToFile(filename="log.json"))
    init()
    main()



from scheduler.core.schemas.structure.task_relation_manager import TaskRelationManager, Node, Direction

if __name__ == '__main__':
    """
    A → B → C
    ↓
    D → F → H
    ↓       ↓
    E → G   I
    """


    class T(Node):
        def __init__(self, s):
            super().__init__()
            self.s = s

        def __str__(self):
            return self.s


    manager = TaskRelationManager()
    # 使用字符串作为任务对象
    task_a = T("A")
    task_b = T("B")
    task_c = T("C")
    task_d = T("D")
    task_e = T("E")
    task_f = T("F")
    task_g = T("G")
    task_h = T("H")
    task_i = T("I")
    task_j = T("J")
    task_k = T("K")
    task_l = T("L")

    manager.set_relationship(task_a, Direction.RIGHT, task_b)
    manager.set_relationship(task_a, Direction.DOWN, task_d)
    manager.set_relationship(task_d, Direction.DOWN, task_e)
    manager.set_relationship(task_d, Direction.RIGHT, task_f)
    manager.set_relationship(task_e, Direction.RIGHT, task_g)
    manager.set_relationship(task_b, Direction.RIGHT, task_c)
    manager.set_relationship(task_f, Direction.RIGHT, task_h)
    manager.set_relationship(task_h, Direction.DOWN, task_i)

    # print(manager.get_neighbor_sub_nodes(task_d))
    # print(manager.remove_node(task_d))
    # for it in manager.get_neighbor_sub_nodes(task_d):
    #     print(it)
    # print(manager.get_upper_chain(task_d, 3))
    # manager.add_sub_tasks(task_i, [task_j, task_k, task_l])
    # for Un in manager.get_upper_chain(task_l, 3):
    #     print(f"from:{Un['from_node']};to:{Un['to_node']};direction:{Un['direction']};dis:{Un['distance_with_start']};")

    manager.draw_graph()



import base64
import pickle


def serialize_args(*args, **kwargs):
    """
    Serialize the output of args and kwargs into bytecode, then return the base64 encoding
    :param args:
    :param kwargs:
    :return: Serialized args and kwargs
    """
    return base64.b64encode(pickle.dumps((args, kwargs))).decode('utf-8')


def deserialize_args(serialized_args):
    """
    Deserialize the serialized args and kwargs
    :param serialized_args:
    :return: Deserialized args and kwargs
    """
    return pickle.loads(base64.b64decode(serialized_args))


def deserialize_args_decorator(func):
    """
    Deserialize the serialized args and kwargs
    :param func:
    :return: A wrapper function that deserializes the args and kwargs
    """

    def wrapper(serialized_args):
        args, kwargs = deserialize_args(serialized_args)
        return func(*args, **kwargs)

    return wrapper



from typing import List

from kink import di
from langchain_core.output_parsers import StrOutputParser
from langchain_core.prompts import ChatPromptTemplate
from langchain_openai import ChatOpenAI

import loguru

from config import Master


def construct_agent_prompt(agent_entry, agent_name, **kwargs):
    """
    Construct agent prompt
    :param agent_entry:
    :param agent_name:
    :return: prompt,agent_args,template_prompt
    """
    agent_work = agent_entry.__doc__ + "\n" + kwargs.get("system_prompt", "")
    kwargs.pop("system_prompt", None)
    agent_args = kwargs
    template_prompt = f"You are {agent_name},your work: {agent_work}"
    template_prompt = Master.get("prefix") + template_prompt
    prompt = ChatPromptTemplate.from_messages([
        ("system", "Your name is {agent_name}. {template_prompt}"),
        ("user", "{input}")
    ])
    return prompt, agent_args, template_prompt


def agent_scheduler(agent_entry, agent_name, **kwargs):
    """
    Call agent with agent_entry and agent_name
    :param agent_entry:
    :param agent_name:
    :return:
    """
    llm = di['llm']
    prompt, agent_args, template_prompt = construct_agent_prompt(agent_entry, agent_name, **kwargs)
    output_parser = StrOutputParser()
    chain = prompt | llm | output_parser
    response = chain.invoke(
        {"input": f"Your args: {agent_args}", "agent_name": agent_name, "template_prompt": template_prompt})
    return response


async def async_agent_scheduler(agent_entry, agent_name, **kwargs):
    """
    Call agent with agent_entry and agent_name
    :param agent_entry:
    :param agent_name:
    :return:
    """
    llm = di['llm']
    prompt, agent_args, template_prompt = construct_agent_prompt(agent_entry, agent_name, **kwargs)
    output_parser = StrOutputParser()
    chain = prompt | llm | output_parser
    response = chain.astream(
        {"input": f"Your args: {agent_args}", "agent_name": agent_name, "template_prompt": template_prompt})
    chunks = []
    async for chunk in response:
        chunks.append(chunk)
        print(chunk, end="")
        if check_closed_function_calls("".join(chunks)):
            break
    return "".join(chunks)


def check_closed_function_calls(string):
    """
    Check if there are closed function calls in the chunks
    :return:
    """
    return True if (string.count("%%") == 2) else False



import base64
import configparser
import hashlib
import hmac
import json
import threading
import time

import requests
from kink import inject


class Manager:
    @inject
    def __init__(self, config: configparser.ConfigParser):
        self.secret = config.get('dingtalk', 'bot_key')
        self.webhook = config.get('dingtalk', 'api_url') + "/robot/send?access_token=" + config.get('dingtalk', 'access_token')

    def send_dtk_msg(self, message):
        """
        从config.py中读取配置
        在反馈群中调用机器人发送信息
        :param message:
        :return:
        """
        timestamp = str(round(time.time() * 1000))
        string_to_sign = f'{timestamp}\n{self.secret}'
        hmac_code = hmac.new(self.secret.encode('utf-8'), string_to_sign.encode('utf-8'), hashlib.sha256).digest()
        signature = base64.b64encode(hmac_code).decode('utf-8')
        signed_url = f'{self.webhook}&timestamp={timestamp}&sign={signature}'
        headers = {'Content-Type': 'application/json; charset=utf-8'}
        data = {
            "msgtype": "text",
            "text": {
                "content": message
            }
        }
        response = requests.post(signed_url, headers=headers, data=json.dumps(data))
        print(response.text)

    def send_message_in_thread(self, message):
        """
        非阻塞运行的发送信息机器人
        功能同send_dingtalk_message
        """
        try:
            thread = threading.Thread(target=self.send_dtk_msg, args=(message,))
            thread.start()
        except Exception as e:
            print(f"发送信息失败: {e}")

    def info(self, message):
        thread = threading.Thread(target=self.send_dtk_msg, args=(f"[信息] {message}",))
        thread.start()

    def warn(self, message):
        thread = threading.Thread(target=self.send_dtk_msg, args=(f"[警告] {message}",))
        thread.start()

    def warn2(self, message):
        thread = threading.Thread(target=self.send_dtk_msg, args=(f"[严重警告] {message}",))
        thread.start()



import functools
import time
import logging

import loguru


def retry(max_retries=3, delay=1):
    """
    重试装饰器

    参数:
    max_retries (int): 最大重试次数。
    delay (int): 重试间隔时间（秒）。
    """

    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            attempts = 0
            while attempts < max_retries:
                try:
                    return func(*args, **kwargs)
                except Exception as e:
                    attempts += 1
                    if attempts >= max_retries:
                        loguru.logger.error(f"Function {func.__name__} failed with {e} after {attempts} attempts.")
                        raise
                    loguru.logger.warning(f"Function {func.__name__} failed with {e}. Retrying in {delay} seconds.")
                    loguru.logger.trace(e)
                    time.sleep(delay)

        return wrapper

    return decorator



# -*- coding: utf-8 -*-
import json
import random

import loguru
import requests

from tools.func.retry_decorator import retry
from config import MCP


class McpClient:
    def __init__(self, mcp_client_base_url=MCP['client']['base_url'],
                 task_id: str = None):
        self.base_url = mcp_client_base_url
        self.task_id = task_id if task_id else 'default_task_id'
        self.new_msg_flag = 'new_msg'

    @retry(max_retries=3, delay=1)
    def execute(self, prompt: str) -> str:
        """
        Execute a prompt using the MCP service with streaming response.
        Returns the final content once the stream is complete.
        """
        loguru.logger.debug(f'McpClient running: {prompt}')
        url = f'{self.base_url}/'

        with requests.post(
                url,
                json={'prompt': prompt, 'mcp_servers': {'kali_driver': MCP['server']['kali_driver'], 'browser_use': MCP['server']['browser_use']}},
                timeout=4 * 60 * 60,  # 4小时超时，适用于长时间任务
                stream=True  # 启用流式传输
        ) as response:
            response.raise_for_status()
            final_content = ''

            for line in response.iter_lines():
                try:
                    data = json.loads(line)
                    current_content = data.get('content', '')
                    final_content += current_content
                    if data.get(self.new_msg_flag, False):
                        # 遇到分隔符重置缓冲区，由于最后一段是没有最后的分隔符的，所以此处缓冲区就会设置为最后一段内容
                        final_content = ''
                    if data.get('done', False):
                        # 停止标记
                        break
                except json.JSONDecodeError:
                    loguru.logger.warning(f"Failed to decode JSON line: {line}")
                    continue

            return final_content


if __name__ == '__main__':
    MC = McpClient('http://10.10.3.119:25989')
    print(MC.execute('告诉我当前的所在网段'))



# ****************************************************************************
# 尝试使用Langchain API调用LLM进行prefill
# ****************************************************************************
from kink import di
from langchain_core.output_parsers import StrOutputParser
from langchain_core.prompts import ChatPromptTemplate
from langchain_openai import ChatOpenAI

from config import Master

if __name__ == '__main__':
    model = di['llm']
    output_parser = StrOutputParser()
    prompt = ChatPromptTemplate.from_messages([
        ("system", "You are a helpful assistant."),
        ("assistant", "{input}"),
        ("user", "Continuously continue")
    ])
    chain = prompt | model | output_parser
    res = chain.invoke({"input": """我可以帮助尝试运行 `ping` 命令以获取相关信息。如果失败，可能仍受到权限限制。开始尝试执行命令：

('\n正在 Ping 100.64.0.41 具有 32 字节的数据:\n来自 100.64.0.41 的回复: 字节=32 时间=392ms TTL=64\n来自 100.64.0.41 的回复: 字节=32 时间=93ms TTL=64\n来自 100.64.0.41 的回复: 字节=32 时间=90ms TTL=64\n来自 100.64.0.41 的回复: 字节=32 时间=101ms TTL=64\n\n100.64.0.41 的 Ping 统计信息:\n    数据包: 已发送 = 4，已接收 = 4，丢失 = 0 (0% 丢失)，\n往返行程的估计时间(以毫秒为单位):\n    最短 = 90ms，最长 = 392ms，平均 = 169ms\n', '', 0)
"""})
    print(res)



import loguru
from langchain.output_parsers import OutputFixingParser
from langchain_core.exceptions import OutputParserException
from langchain_core.language_models import BaseChatModel
from langchain_core.output_parsers import PydanticOutputParser
from langchain_core.prompts import ChatPromptTemplate
from langchain_core.runnables.utils import Input
from pydantic import BaseModel
from tools.logging import logging
from tools.args_wrap.loading import running_indicator


@logging.function_logging(logging_level=0)
def chat_with_safety_pydantic_output_parser(model: BaseChatModel, input_args: Input, promptTemplate: ChatPromptTemplate,
                                            schemas_model: BaseModel) -> BaseModel:
    """
    Chat with Safety Pydantic Output Parser
    :param model:
    :param input_args:
    :param promptTemplate:
    :param schemas_model:
    :return: Pydantic Model

    Example:
            model = ChatOpenAI(
            temperature=0.0,
            model=Master.get("default_model"),
            base_url=Master.get("openai_api_endpoint"),
            api_key=Master.get("openai_api_key"),
        )
        parser = PydanticOutputParser(pydantic_object=TaskStatusModel)
        promptTemplate = ChatPromptTemplate.from_messages([
            ("system", "你是一名助手，请根据用户的问题和另一位工人的执行结果综合判断此任务状态如何，返回格式请严格遵循以下要求{format_instructions};"
                       "(不要尝试去实际执行任务!)"
                       "你有权限调用一些函数，另一位工人和你有同等权限，这有助于你判断其状态，下文会给你函数列表;"
             ),
            ("user", "任务简述:```{abstract}```;任务描述:```{description}```;执行结果:```{result}```;验收标准:{verification}"
                     "函数列表:{func_list}")
        ])
        func_list = self.tools_manager.list_func_json()
        input_args = {
            "format_instructions": parser.get_format_instructions(),
            "abstract": self.abstract,
            "description": self.description,
            "func_list": str(func_list),
            "result": result,
            "verification": self.verification,
        }
        task_status_model = ChatWithSafetyPydanticOutputParser(model=model, input_args=input_args,
                                                               promptTemplate=promptTemplate,
                                                               schemas_model=TaskStatusModel)
    """
    parser = PydanticOutputParser(pydantic_object=schemas_model)
    chain = promptTemplate | model

    raw_output = chain.invoke(input_args)
    logging.log(data={
        "raw_output": raw_output,
        "input_args": input_args
    })
    try:
        raw_output = str(raw_output.content)
        schemas_model = parser.parse(raw_output)
    except OutputParserException as e:
        try:
            schemas_model = OutputFixingParser.from_llm(parser=parser, llm=model).parse(raw_output)
        except OutputParserException as e:
            return chat_with_safety_pydantic_output_parser(model=model, input_args=input_args,
                                                           promptTemplate=promptTemplate, schemas_model=schemas_model)
    return schemas_model



from typing import Dict, List

import kink
import loguru
from kink import inject, di
from langchain_core.output_parsers import PydanticOutputParser
from langchain_core.prompts import ChatPromptTemplate

from scheduler.core.Thought import Thought
from scheduler.core.mcp_client.mcp_client import McpClient
from scheduler.core.schemas.schemas import TaskModel, NeedBranchModel, TaskExecuteStatusModel, \
    TaskModelOut, TaskStatus
from scheduler.core.schemas.structure.ToT import TaskObject
from scheduler.core.schemas.structure.task_relation_manager import Node, TaskRelationManager, Direction
from scheduler.core.schemas.works.PydanticSafetyParser import chat_with_safety_pydantic_output_parser
from scheduler.core.tasks.exceptions.task_exceptions import TaskNeedTurningException, TaskImpossibleException
from tools.func.retry_decorator import retry
from tools.logging import logging


class TaskNode(Node):
    @kink.inject
    def __init__(
            self,
            task_model: TaskModel,
            trm: TaskRelationManager,
            mcp_client: McpClient,
            graph_name: str = 'default_graph_name',
            taskId: str = None,
    ):
        """
        Task class's init func.
        :param task_model: TaskModel
        :param trm: TRM obj.
        """

        super().__init__()
        self.task_pydantic_model = TaskObject(
            task_model=task_model,
            task_out_model=None,
            task_status_model=TaskStatus.PENDING
        )
        self.taskId = taskId
        self._trm = trm
        self.task = task_model
        self.mcp_client = mcp_client
        self.abstract = task_model.abstract
        self.description = task_model.description
        self.verification = task_model.verification
        loguru.logger.debug(f"Task: `{self.abstract}` has been created.")
        self._trm.add_task(self)
        self.graph_name = graph_name

        self._replan_counter = 0

    def __str__(self):
        return f"Task:{self.task_pydantic_model}\n"

    def _flush_graph(self):
        """
        Flush the graph.
        :return:
        """
        self._trm.draw_graph(self.graph_name)

    @logging.function_logging(-1)
    def branch_and_execute(self, branch_requirement) -> TaskModelOut:
        """
        The worker need to do the branch task.
        :return:
        """

        task_chain = branch_requirement.task_chain

        tasks_classed = []
        task_chain_output: TaskModelOut | None = None
        for subtask in task_chain.tasks:
            subtask = TaskNode(task_model=subtask, trm=self._trm, mcp_client=self.mcp_client,
                               graph_name=self.graph_name)
            tasks_classed.append(subtask)
        self._trm.add_sub_tasks(current_task=self, sub_task=tasks_classed)
        for subtask in tasks_classed:
            try:
                # 一个链上的任务中最后一个节点的结果是整个链的最终结果
                # 所以栈中的任务结果依次覆盖，最终出循环时，结果就是最后一个节点的结果
                task_chain_output = subtask.execute()
            except TaskImpossibleException as e:
                if self._replan_counter > 3:
                    raise e

                loguru.logger.warning(f"Task {self.task_pydantic_model} is impossible, replan it.")
                self._replan_counter += 1
                # 重新规划任务并执行, 这里只需要直接重新调用execute, ai会规划的
                self.execute(f'任务:{self.task_pydantic_model}执行失败，失败原因:{e}，需要更改任务规划')
            except Exception as e:
                raise e
        return task_chain_output

    @logging.function_logging(-1)
    def direct_execute(self, advices, articles) -> TaskModelOut:
        """
        The worker do the task.
        :return:
        """
        loguru.logger.info(f"Task {self.task_pydantic_model} is working, articles: {articles}")
        self.task_pydantic_model = self.task_pydantic_model.copy(update={
            "task_status_model": TaskStatus.WORKING
        })

        max_try = 3
        for i in range(max_try):
            try:
                result = self.run_mcp_agent(articles=articles, advices=advices)
                if self.check_task_result(result):
                    result: TaskModelOut = self.digest_result_to_abstract(result=result)
                    self.task_pydantic_model = self.task_pydantic_model.copy(update={
                        "task_status_model": TaskStatus.SUCCESS,
                        "task_out_model": result
                    })
                    loguru.logger.success(f"Task {self.task_pydantic_model} is successful, result: {result}")
                    return result
            except TaskNeedTurningException as e:
                advices += f"此任务你已经尝试过了，但是没有成功，以下是给此次执行的建议:{e}"
            except TaskImpossibleException as e:
                self._trm.remove_node(self)
                raise e
            except Exception as e:
                self._trm.remove_node(self)
                raise e
        self._trm.remove_node(self)
        raise TaskImpossibleException(f"此任务已经尝试{max_try}次了，均没有成功")

    def execute(self, rebranch_prompt='') -> TaskModelOut:
        """
        The task's core.
        There are lots of thoughts in the villager.
        :return:
        """
        loguru.logger.warning(f'task_id: {self.id}')
        articles = ''
        advices = ''
        upper_chain: List[Node] = self._trm.get_upper_import_node_simple(self, window_n=3, window_m=6)

        if len(upper_chain) > 0:
            # 含有上级或平级的前置任务
            advices = f'作为你的执行参考，你的上级或平级的前置任务如下(仅供参考，不要执行):'  # 覆盖
            upper_chain.reverse()  # 栈序翻转
            for upper_node in upper_chain:
                advices += f'\n{upper_node.task_pydantic_model}'
        advices += f'\n{rebranch_prompt}'

        branch_requirement: NeedBranchModel = self.check_branching_requirement(advice=advices)
        self._flush_graph()
        if len(branch_requirement.task_chain.tasks) > 1:
            try:
                return self.branch_and_execute(branch_requirement)
            except TaskImpossibleException as e:
                # 若下级任务产生任务不可能的错误，在此级捕获并重新分配任务分支
                return self.execute()
        else:
            return self.direct_execute(advices, articles)

    @retry(max_retries=5, delay=1)
    @inject
    @logging.function_logging(-1)
    def digest_result_to_abstract(self, result: str, llm):
        """
        Focus on summary of mission results.
        :return:
        """
        pydantic_object = TaskModelOut
        model = llm
        parser = PydanticOutputParser(pydantic_object=pydantic_object)
        promptTemplate = ChatPromptTemplate.from_messages([
            ("system", "{format_instructions};"
                       "你是一名摘要员，负责将下文的结果报告摘要为有价值的(task所关注的)内容，返回格式请严格遵循以上要求;"
                       "只允许摘要文章中出现过的事实内容, 不允许添加任何假设或二级推断的内容;"
                       "(不要尝试去实际执行此任务!)"
             ),
            ("user", "结果报告:{result_report};此结果的对应任务:{task}")
        ])
        input_args = {"result_report": result,
                      "task": self.task,
                      "format_instructions": parser.get_format_instructions(),
                      }
        return chat_with_safety_pydantic_output_parser(model=model, input_args=input_args,
                                                       promptTemplate=promptTemplate,
                                                       schemas_model=pydantic_object)

    @retry(max_retries=5, delay=1)
    @inject
    @logging.function_logging(-1)
    def check_branching_requirement(self, llm, advice=''):
        """
        The thought think about do we need branch for this task.
        :param llm: Dependency Injection's llm object
        :param advice:
        :return:
        """
        pydantic_object = NeedBranchModel
        model = llm
        parser = PydanticOutputParser(pydantic_object=pydantic_object)
        promptTemplate = ChatPromptTemplate.from_messages([
            ("system", "{format_instructions};"
                       "你是一名规划师，请根据用户的问题综合判断，我们是否需要细分该任务才能完成这个任务"
                       "(不要尝试去实际执行任务!)"
                       "如果需要，按照顺序提供任务链，并保证任务链的连续性，让上一个任务的结果可以直接被下一个任务使用，如果不需要，请提供一个长度为1的任务链。"
                       "每个任务仅在简单到只包含一个具体动作且前置信息充分时才能被认为不需要被细分"
             ),
            ("user",
             "任务简述:{abstract};任务描述:{description};作为你的执行参考，你的上级或平级的前置任务如下(仅供参考，不要执行):{advice};")
        ])
        input_args = {"abstract": self.abstract,
                      "description": self.description,
                      "format_instructions": parser.get_format_instructions(),
                      "advice": advice,
                      }
        res = chat_with_safety_pydantic_output_parser(model=model, input_args=input_args,
                                                      promptTemplate=promptTemplate,
                                                      schemas_model=pydantic_object)
        loguru.logger.debug(f"Task chain {res}")
        return res

    @logging.function_logging(-1)
    def run_mcp_agent(self, articles: str = '', advices: str = '',
                      prompt='请你帮我完成以下任务，并返回应返回的信息，在过程中请遵从事实，不要假设，以下是需要完成的内容:'):
        return self.mcp_client.execute(
            f'{prompt}任务摘要:{self.abstract}\n'
            f'任务描述:{self.description}\n'
            f'{articles};{advices};')

    @logging.function_logging(-1)
    def check_task_result(self, result: str):
        """
        Check the task's result is correct or not.
        :return:
        """
        pydantic_object = TaskExecuteStatusModel
        model = di['llm']
        parser = PydanticOutputParser(pydantic_object=pydantic_object)
        promptTemplate = ChatPromptTemplate.from_messages([
            ("system", "你是一名助手，请根据用户的问题和另一位工人的执行结果综合判断此任务状态如何，返回格式请严格遵循以下要求{format_instructions};"
                       "请注意:"
                       "1. 不要尝试去实际执行任务!"
                       "2. 你有权限调用一些函数，另一位工人和你有同等权限，这有助于你判断其状态，下文会给你函数列表;"
             ),
            ("user",
             "任务简述:```{abstract}```;任务描述:```{description}```;执行结果:```{result}```;验收标准:{verification}")
        ])
        input_args = {
            "format_instructions": parser.get_format_instructions(),
            "abstract": self.abstract,
            "description": self.description,
            "result": result,
            "verification": self.verification,
        }
        task_status_model = chat_with_safety_pydantic_output_parser(model=model, input_args=input_args,
                                                                    promptTemplate=promptTemplate,
                                                                    schemas_model=pydantic_object)
        if task_status_model.is_task_successful == 0:
            if task_status_model.is_task_impossible == 0:
                raise TaskNeedTurningException(task_status_model.explain)
            else:
                explain_str = f"任务:{self.abstract}执行失败，失败原因:{task_status_model.explain}"
                # 只有不可能的任务才会向父任务抛出异常，所以需要明确任务简述
                raise TaskImpossibleException(explain_str)
        else:
            return True



from enum import Enum

import loguru
from playwright.sync_api import sync_playwright, Playwright


class WaitUntilState(Enum):
    LOAD = 'load'
    DOM_CONTENT_LOADED = 'domcontentloaded'
    NETWORK_IDLE = 'networkidle'
    COMMIT = 'commit'


class FILE_TYPE(Enum):
    JPG: str = 'jpg'
    PNG: str = 'png'
    GIF: str = 'gif'
    PDF: str = 'pdf'
    DOC: str = 'doc'
    DOCX: str = 'docx'
    XLS: str = 'xls'
    XLSX: str = 'xlsx'
    PPT: str = 'ppt'
    PPTX: str = 'pptx'
    TXT: str = 'txt'
    ZIP: str = 'zip'
    RAR: str = 'rar'
    GZ: str = 'gz'
    TAR: str = 'tar'
    BZ2: str = 'bz2'
    Z: str = 'z'
    TARGZ: str = 'tar.gz'


class CrawlerBase:
    history_urls: list[str] = []
    thread_count: int = 1
    black_list_file_type: list[FILE_TYPE] = [FILE_TYPE.JPG, FILE_TYPE.PNG, FILE_TYPE.GIF, FILE_TYPE.PDF, FILE_TYPE.DOC,
                                             FILE_TYPE.DOCX, FILE_TYPE.XLS, FILE_TYPE.XLSX, FILE_TYPE.PPT,
                                             FILE_TYPE.PPTX, FILE_TYPE.TXT, FILE_TYPE.ZIP, FILE_TYPE.RAR, FILE_TYPE.GZ,
                                             FILE_TYPE.TAR, FILE_TYPE.BZ2, FILE_TYPE.Z, FILE_TYPE.TARGZ]


class Crawler(CrawlerBase):

    def __init__(self, url):
        self.url = url

        # Initialize the playwright browser
        self.playwright = sync_playwright().start()

    def get_page_content(self, **kwargs):
        """
        Get the page content
        :return: Raw HTML content
        """
        loguru.logger.debug("Getting page content...")
        browser = self.playwright.chromium.launch()
        page = browser.new_page()
        page.goto(self.url, **kwargs)
        content = page.content()
        browser.close()
        loguru.logger.debug("Page content obtained.")
        return content

    def get_page_text(self, **kwargs):
        """
        Get the full text content of the rendered page
        :return: Full text content
        """
        waiting_mode = WaitUntilState.NETWORK_IDLE
        loguru.logger.debug("Getting page text...")
        browser = self.playwright.chromium.launch()
        page = browser.new_page()
        page.goto(self.url, wait_until="networkidle")
        text_content = page.inner_text('body')
        browser.close()
        loguru.logger.debug("Page text obtained.")
        return text_content

    def crawler_for_domain(self, domain_list: list[str]):
        """
        Crawler,enter from url,and crawl the domain_list
        :param domain_list: List of domains, e.g. ['example.com', 'example2.com']
        :return:
        """
        ...


if __name__ == '__main__':
    crawler = Crawler("http://100.64.0.33")
    content = crawler.get_page_content(wait_until="networkidle")
    loguru.logger.info(content)



import uuid
from collections import deque
from enum import auto, Enum
from typing import List, Dict

from tools.moveptr.pairwise import pairwise


class Direction(Enum):
    UP = auto()
    DOWN = auto()
    LEFT = auto()
    RIGHT = auto()


def get_reverse_direction(direction: Direction):
    if direction == Direction.UP:
        return Direction.DOWN
    elif direction == Direction.DOWN:
        return Direction.UP
    elif direction == Direction.LEFT:
        return Direction.RIGHT
    elif direction == Direction.RIGHT:
        return Direction.LEFT
    else:
        return None


class node:
    def __init__(self):
        unique_id = uuid.uuid4().int
        self.id = unique_id & 0xFFFFFFFF


class TaskRelationManager:
    def __init__(self):
        """
        This is a class that records the memory of the Tree of Tasks.
        It can record task's relationships and their properties.

        Every task had a unique task class (task.py -> task)
        Every task had a UP DOWN LEFT RIGHT node(task),of course ,the relationship node can be null.
        """
        self.task_registry = {}
        self.relationships = {}

    def _get_task_from_id(self, id: int):
        """
        Get task obj from id
        :param id:
        :return:
        """
        if id in self.task_registry:
            return self.task_registry[id]

    def _get_task_id(self, task: node) -> int:
        """内部方法：获取任务的唯一ID"""
        if task:
            task_id = task.id
            if task_id not in self.task_registry:
                self.task_registry[task_id] = task

            return task_id

    def add_task(self, task: node) -> int:
        """
        注册新任务
        :param task: 任何可哈希的对象（建议使用不可变对象）
        :return: 生成的唯一任务ID
        """
        task_id = self._get_task_id(task)
        if task_id not in self.relationships:
            self.relationships[task_id] = {
                Direction.UP: None,
                Direction.DOWN: None,
                Direction.LEFT: None,
                Direction.RIGHT: None
            }
        return task_id

    def is_neighbors(self, n1: node, n2: node) -> bool:
        """
        Is n1 neighbor n2?
        :param n1:
        :param n2:
        :return:
        """
        if n1 and n2:
            n1_id = self._get_task_id(n1)
            n2_id = self._get_task_id(n2)
            n1_n = self.get_neighbors(n1)
            if n2_id in n1_n.values():
                return True
        return False

    def get_neighbor_direction(self, current_n: node, target_n: node):
        """
        Get neighbor's direction.
        :param current_n:
        :param target_n:
        :return:
        """
        if self.is_neighbors(current_n, target_n):
            for d in Direction:
                if self.get_direction_neighbors(current_n, d) == self._get_task_id(target_n):
                    return d

    def get_neighbor_sub_nodes(self, current_n: node) -> List[node]:
        """
        Get sub nodes(right and down).
        :param current_n:
        :return: List of node.
        """
        res = []
        for d in [Direction.DOWN, Direction.RIGHT]:
            Sn = self.get_direction_neighbors(current_n, d)  # Sub node
            if Sn:
                Sn = self._get_task_from_id(Sn)
                res.append(Sn)
        return res

    def unlink(self, n1: node, n2: node) -> bool:
        """
        Dissolve the relationship between n1 and n2
        :param n1:
        :param n2:
        :return:
        """
        d = self.get_neighbor_direction(n1, n2)
        n1_id = self._get_task_id(n1)
        n2_id = self._get_task_id(n2)
        if d:
            self.relationships[n1_id][d] = None
            self.relationships[n2_id][get_reverse_direction(d)] = None
            return True
        return False

    def remove_node(self, task: node) -> int:
        """
        Remove node and remove theis sub nodes.
        :param task:
        :return:
        """

        for Sn in self.get_neighbor_sub_nodes(task):
            if self.get_neighbor_sub_nodes(Sn):
                self.remove_node(Sn)
            self.unlink(task, Sn)

    def _get_available_sub_direction(self, current_node: node):
        """
        Return a empty direction.
        :param current_node:
        :return:
        """
        for d in [Direction.RIGHT, Direction.DOWN]:
            if not self.get_direction_neighbors(current_node, d):
                return d

    def add_sub_tasks(self, current_task: node, sub_task: List[node]):
        """
        Create new sub_task_chain and auto set empty direction.
        :param current_task:
        :param sub_task:
        :return:
        """
        d = self._get_available_sub_direction(current_task)
        chain = [current_task]
        chain.extend(sub_task)
        if d:
            # 一般来说输入的结构正确不会有sub direction完全占满的状态，但这个仅为了保险
            for n1, n2 in pairwise(chain):
                self.set_relationship(n1, d, n2)

    def set_relationship(self, from_task: node,
                         direction: Direction,
                         to_task: node = None):
        """
        设置任务间关系（None表示解除关系）
        :param from_task: 关系发起方任务
        :param direction: Direction枚举值
        :param to_task: 关系接收方任务
        """
        from_id = self._get_task_id(from_task)
        to_id = self._get_task_id(to_task) if to_task else None

        if from_id not in self.relationships:
            self.add_task(from_task)
        if to_task and to_id not in self.relationships:
            self.add_task(to_task)

        self.relationships[from_id][direction] = to_id
        self.relationships[to_id][get_reverse_direction(direction)] = from_id

    def get_task_chain(self, start_task: node,
                       direction: Direction) -> list:
        """
        获取特定方向的任务链
        :param start_task: 起始任务
        :param direction: 遍历方向
        :return: 任务ID链表
        """
        chain = []
        current_id = self._get_task_id(start_task)
        while current_id:
            chain.append(current_id)
            current_id = self.relationships.get(current_id, {}).get(direction)
        return chain

    def get_neighbors(self, task: node) -> dict:
        """
        获取任务的四向邻居
        :param task: 目标任务
        :return: 包含四个方向任务ID的字典
        """
        task_id = self._get_task_id(task)
        return self.relationships.get(task_id, {})

    def get_direction_neighbors(self, task: node, direction: Direction) -> int:
        """
        获取指定方向的邻居
        :param task: 目标任务
        :param direction: 遍历方向
        :return: 任务ID链表
        """
        task_id = self._get_task_id(task)
        return self.relationships.get(task_id, {}).get(direction)

    def get_upper_chain(self, start_task: node, window_len: int) -> List[Dict]:
        """
        获取指定节点的上级链路的最近window_len个对象。
        同一层级优先收集上级节点，其次左节点，按层级顺序返回，直到达到window_len的数量。

        :param start_task: 起始任务节点
        :param window_len: 需要返回的上级节点数量
        :return: 包含任务对象的列表，按从近到远顺序排列，每个元素包含方向和距离
        """
        if window_len <= 0:
            return []
        result = []

        def add_node(to_node, from_node, direction, distance):
            result.append({
                "from_node": from_node,
                "to_node": to_node,
                "direction": direction,
                "distance_with_start": distance
            })

        up_node = self._get_task_from_id(self.get_direction_neighbors(start_task, Direction.UP))
        left_node = self._get_task_from_id(self.get_direction_neighbors(start_task, Direction.LEFT))

        if up_node:
            add_node(up_node, start_task, Direction.UP, 1)
        if left_node:
            add_node(left_node, start_task, Direction.LEFT, 1)
        if window_len == 1:
            return result
        elif window_len > 1:
            window_len -= 1
            up_chain_from_left = self.get_upper_chain(left_node, window_len)
            up_chain_from_up = self.get_upper_chain(up_node, window_len)
            # 把这两个链里面的distance ++并保存回链
            for item in up_chain_from_left:
                item["distance_with_start"] += 1
                result.append(item)
            for item in up_chain_from_up:
                item["distance_with_start"] += 1
                result.append(item)
            return result




import loguru
from kink import inject, di
import requests


class checkEnv:
    def __init__(self, min_memory=256, need_camera=False):
        self.min_memory = min_memory
        self.need_camera = need_camera
        try:
            loguru.logger.warning("开始环境检查")
            loguru.logger.debug('-' * 32)
            self.checkCamera()
            self.checkMemory()
            self.checkNetwork()
            loguru.logger.debug('-' * 32)
            loguru.logger.success("环境检查通过")
        except Exception as e:
            loguru.logger.debug('-' * 32)
            loguru.logger.error("环境检查失败")
            exit(0)

    @inject
    def checkNetwork(self, proxy: str):
        """
        检查网络环境是否正常，支持代理。

        :param proxy: 代理服务器的 URL
        :return: 网络环境是否正常
        """
        # 先检查一下dns，看能不能解析百度地址
        import socket
        try:
            ip = socket.gethostbyname("www.baidu.com")
            loguru.logger.info(f"DNS: www.baidu.com -> {ip}")
        except Exception as e:
            loguru.logger.error("DNS解析失败，请检查网络环境")
            raise Exception("DNS解析失败，请检查网络环境")
        loguru.logger.success("DNS正常")

        try:
            requests.get("http://www.baidu.com")
            loguru.logger.success("通联国内互联网正常")
        except Exception as e:
            raise Exception("实际通联网络环境测试异常")
        try:
            if proxy:
                loguru.logger.info('使用代理')
                cn_res = requests.get("http://www.baidu.com", proxies={"http": proxy, "https": proxy})
                loguru.logger.success("通联国内互联网代理正常")
                if len(cn_res.content) < 1:
                    loguru.logger.error(f"通联国内互联网代理异常 length:{len(cn_res.content)}")
                    raise Exception("通联国内互联网代理异常")
                res = requests.get("http://www.google.com", proxies={"http": proxy, "https": proxy})
                if len(res.content) < 1:
                    loguru.logger.error(f"通联国外互联网代理异常 length:{len(res.content)}")
                    raise Exception("通联国外互联网代理异常")
                loguru.logger.success(f"通联国外互联网代理正常 length:{len(res.content)}")
        except Exception as e:
            loguru.logger.error("通联网络环境代理测试异常")
            raise Exception("实际通联网络环境代理测试异常")
        # 获取网卡信息
        import psutil
        net = psutil.net_if_addrs()
        for k, v in net.items():
            for item in v:
                if item.family == 2:
                    loguru.logger.info(f"Network: {k} {item.address}")
                    break
        loguru.logger.success("网络环境正常")

    def checkCamera(self):
        if self.need_camera:
            import cv2
            cap = cv2.VideoCapture(0)
            if not cap.isOpened():
                loguru.logger.error("摄像头无法打开")
                raise Exception("摄像头无法打开")
            # 获取摄像头的分辨率和总共的摄像头数量
            width = cap.get(cv2.CAP_PROP_FRAME_WIDTH)
            height = cap.get(cv2.CAP_PROP_FRAME_HEIGHT)
            # 获取系统中摄像头的总数量
            count = 0
            while True:
                test_cap = cv2.VideoCapture(count)
                if not test_cap.isOpened():
                    break
                test_cap.release()
                count += 1
            loguru.logger.info(f"Camera: {width}x{height}, {count} cameras")
            # 拍摄一张图片
            loguru.logger.info("正在测试摄像头...")
            ret, frame = cap.read()
            if not ret or frame is None:
                loguru.logger.error("摄像头无法拍摄图片")
                raise Exception("摄像头无法拍摄图片")
            loguru.logger.success("摄像头正常")
            cap.release()  # 释放摄像头

    def checkMemory(self):
        import psutil
        memory = psutil.virtual_memory().total / 1024 / 1024
        current_used_memory = psutil.virtual_memory().used / 1024 / 1024
        if memory-current_used_memory < self.min_memory:
            loguru.logger.error(f"内存不足{self.min_memory}MB，当前剩余内存{memory - current_used_memory}MB")
            raise Exception("内存不足")
        used_rate = current_used_memory / memory * 100
        loguru.logger.info(f"Memory: {current_used_memory}/{memory}MB {used_rate}%")
        loguru.logger.success("内存正常")


if __name__ == '__main__':
    di["proxy"] = "https://huancun:ylq123..@home.hc26.org:5422"
    checkEnv(need_camera=True, min_memory=256)



def format_anchor_message(anchors):
    """
    锚点离线文字的格式化函数
    :param anchors:
    :return:
    """
    if anchors:
        message_lines = [f"锚点 {anchor.id} - {anchor.location} 离线" for anchor in anchors]
        return "\n".join(message_lines)
    else:
        return "当前无离线锚点。"

# list去重函数
def list_unique(input_list):
    """
    列表去重函数
    :param input_list:
    :return:
    """
    return list(set(input_list))


import ipaddress


def cidr_to_ip_list(cidr):
    """
    将 CIDR 范围转换为 IP 地址列表
    :param cidr: str, 例如 '0.0.0.0/24'
    :return: list, 包含所有 IP 地址的字符串列表
    """
    try:
        network = ipaddress.ip_network(cidr)
        return [str(ip) for ip in network.hosts()]
    except ValueError as e:
        raise ValueError(f"无效的CIDR格式: {cidr}，错误: {e}")



from enum import Enum
from typing import List

from pydantic import BaseModel, Field


class NeedRAGModel(BaseModel):
    isNeed: int = Field(description="是否需要查询资料库，1为需要，0为不需要")
    keywords: str = Field(description="空格隔开的关键词")


class TaskStatus(Enum):
    PENDING = "pending"
    WORKING = "working"
    ERROR = "error"
    SUCCESS = "success"

    def __str__(self):
        """
        Let the class can be JSON serialized.
        :return:
        """
        return self.value


class TaskModel(BaseModel):
    abstract: str = Field(description="任务简述")
    description: str = Field(description="任务完整描述")
    verification: str = Field(description="任务验证的标准（例如：是否提供了包含漏洞的返回包）")


class TaskModelOut(BaseModel):
    result_abstract: str = Field(description="任务执行结果的摘要")
    result: str = Field(description="任务执行结果的详细信息")


class TaskChainModel(BaseModel):
    tasks: List[TaskModel] = Field(description="任务列表")


class NeedBranchModel(BaseModel):
    task_chain: TaskChainModel = Field(description="单任务节点或任务节点链")


class TaskExecuteStatusModel(BaseModel):
    is_task_successful: int = Field(description="此任务是否成功完成，1为成功，0为不成功")
    is_task_impossible: int = Field(
        description="如果没有成功完成，此任务以你的能力是否不可能完成，1为不可能，0为可能，请勿轻易返回不可能。")
    explain: str = Field(
        description="如果不可能完成，解释不可能完成的原因。如果可能完成，说明任务应该的执行方式问题在哪，如何修正")



def append__doc__(func):
    """
    Append the docstring of the function to the return key
    :param func:
    :return:
    """

    def wrapper(*args, **kwargs):
        result = func(*args, **kwargs)
        return result, func.__doc__

    return wrapper



from __future__ import annotations

from time import sleep
from functools import wraps
from rich.progress import Progress, SpinnerColumn, TextColumn, TaskID
from rich.console import Console
import threading

console = Console()

progress: Progress | None = None

tasks = []
running = False


def __worker():
    global progress
    # 模拟并行任务
    while running:
        for task in tasks:
            progress.update(task)


def start(flush_time=1):
    global progress
    global running
    running = True
    if progress is None:
        progress = Progress(
            SpinnerColumn(),  # 加载动画
            TextColumn("[progress.description]{task.description}"),  # 任务描述
            transient=True,  # 完成后隐藏进度条
        )
        progress.start()


# threading.Thread(target=__worker, args=()).start()

def stop():
    global running
    running = False


def add_task(title):
    global tasks
    global progress
    task = progress.add_task(title, total=None)
    tasks.append(task)
    return task


def done_task(task: TaskID):
    progress.remove_task(task)
    global tasks
    tasks = list(filter(lambda x: x != 3, tasks))


def running_indicator(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        task_id = add_task(f"[bold green]Running {func.__name__}...")
        try:
            result = func(*args, **kwargs)
            return result
        except Exception as e:
            console.print(f"[red]Error in {func.__name__}: {e}")
            raise
        finally:
            done_task(task_id)

    return wrapper


start()


import logging

import requests


def get_current_ip():
    """
    多种方式稳定的获取当前IP
    :return:
    """
    try:
        # 通过请求api.ipify.org获取
        ip = requests.get('https://api.ipify.org',timeout=10).text.strip()
        return ip.replace('\n', '')
    except Exception as e:
        logging.error(f"请求api.ipify.org失败 {e}")
    try:
        # 通过请求httpbin.org获取
        ip = requests.get('http://httpbin.org/ip',timeout=10).json()['origin']
        return ip.replace('\n', '')
    except Exception as e:
        logging.error(f"请求httpbin.org失败 {e}")
    return None


# print(get_current_ip())



import asyncio

from scheduler.agent_scheduler_manager import agent_scheduler, async_agent_scheduler


def tool_villager(agent_name="实用的助手", **kwargs):
    """
    现在你的系统接入了MCP(Model Context Protocol)，这可以使你和外部函数之间建立了沟通的桥梁，这大大扩展了你的能力。 调用方式：把函数调用的json语句用双百分号括起来当作变量把工具调用方式嵌入形如%%{
    "name": "...", "parameters": {"a": 1,...}}%%的调用语句，其中json含有必需的name和parameters字段。
    例如: Q: 256+1024等于多少 A: 等于%%{"name": "add", "parameters": {"n1": 256, "n2": 1024}}%%。
    """
    resp = agent_scheduler(agent_entry=tool_villager, agent_name=agent_name, **kwargs)
    return resp


async def async_tool_villager(agent_name="实用的助手", **kwargs):
    """
    你作为一个助手，所有思维都请清晰的表达出来，同时现在你有了使用外界部分函数的权限，如果要使用请保证符合要求的使用，使用方法：把函数调用的json语句用双百分号括起来当作变量把工具调用方式嵌入即可。
    如需调用，直接在回复中自然的插入形如%%{"name": "...", "parameters": {"a": 1,...}}%%的调用语句。
    例如:
    Q: 256+1024等于多少
    A: 等于%%{"name": "add", "parameters": {"n1": 256, "n2": 1024}}%%。
    """
    resp = await async_agent_scheduler(agent_entry=tool_villager, agent_name=agent_name, streaming=True, **kwargs)


async def main():
    await async_tool_villager(input="帮我ping一下100.64.0.41")


if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



# -*- coding: utf-8 -*-
import asyncio
import os
import traceback
import uuid
from typing import Dict
from fastapi import FastAPI

from scheduler.core.init import global_llm
from scheduler.core.mcp_client.mcp_client import McpClient
from scheduler.core.schemas.schemas import TaskModel
from scheduler.core.schemas.structure.task_relation_manager import TRM
from scheduler.core.tasks.task import TaskNode
from config import MCP


mcp_client_baseurl = MCP['client']['base_url']



class Task:
    def __init__(self, token, status, graph_context, abstract, description, verification, context):
        self.token = token
        self.status = status

        self.graph_context = graph_context
        self.abstract = abstract
        self.description = description
        self.verification = verification
        self.context = context

    def to_dict(self):
        return {
            'token': self.token,
            'status': self.status,
            'graph_context': self.graph_context,
            'abstract': self.abstract,
            'description': self.description,
            'verification': self.verification,
        }

    def set_graph(self, graph):
        self.graph_context = graph
        return self


# 全局任务状态管理器
task_manager: Dict[str, dict] = {}


def boot(app: FastAPI):
    # 创建任务并后台运行
    async def run_background_task(task_id, abstract, description, verification):
        graph_name = f'./{task_id}.mermaid'

        def blocking_task():
            try:
                # 检查任务是否已被取消
                if task_manager[task_id]['status'] == 'cancelled':
                    return

                with global_llm():
                    with TRM() as trm:
                        # 检查任务是否已被取消
                        if task_manager[task_id]['status'] == 'cancelled':
                            return
                        mcp_client = McpClient(mcp_client_base_url=mcp_client_baseurl, task_id=task_id)
                        _task = TaskNode(
                            task_model=TaskModel(abstract=abstract, description=description, verification=verification),
                            trm=trm,
                            mcp_client=mcp_client,
                            graph_name=graph_name
                        )
                        task_manager[task_id]['mcp_client'] = mcp_client
                        # 检查任务是否已被取消
                        if task_manager[task_id]['status'] == 'cancelled':
                            return

                        result = _task.execute()

                        # 检查任务是否已被取消
                        if task_manager[task_id]['status'] == 'cancelled':
                            return

                        task_manager[task_id]['status'] = 'completed'
                        task_manager[task_id]['result'] = result

            except Exception as e:
                print(f"Background task {task_id} failed: {e}")
                traceback.print_exc()
                task_manager[task_id]['status'] = 'failed'

        task_manager[task_id] = {
            "status": "running",
            "graph_name": graph_name,
            "abstract": abstract,
            "description": description,
            "verification": verification,
        }
        try:
            await asyncio.to_thread(blocking_task)
        except Exception as e:
            print(f"Background task {task_id} failed: {e}")
            traceback.print_exc()

    @app.post("/task")
    async def post_task(abstract: str, description: str, verification: str):
        task_id = str(uuid.uuid4().hex)
        # 启动后台任务
        asyncio.create_task(run_background_task(task_id, abstract, description, verification))
        return {"task_id": task_id}

    @app.get("/get/task/status")
    async def get_task_status():
        tasks = []
        for key in task_manager.keys():
            graph_name = task_manager[key]['graph_name']
            graph_context = ""
            if os.path.exists(graph_name):
                with open(graph_name, 'r') as f:
                    graph_context = f.read()
            tasks.append(Task(token=key, status=task_manager[key]["status"], graph_context=graph_context,
                              abstract=task_manager[key]["abstract"], description=task_manager[key]["description"],
                              verification=task_manager[key]["verification"],
                              context=""))
        return tasks

    @app.get("/tree")
    async def get_tree(task_id: str):
        task_info = task_manager.get(task_id)
        if not task_info:
            return {"error": "Task not found"}

        graph_name = task_info['graph_name']
        if not os.path.exists(graph_name):
            return {"status": task_info['status'], "message": "Graph file not ready yet."}

        with open(graph_name, 'r') as f:
            content = f.read()
        return {"status": task_info['status'], "graph": content}

    @app.get('/task/{task_id}/tree')
    async def get_tree_v1(task_id: str):
        return get_tree(task_id=task_id)

    @app.put("/task/{task_id}/stop")
    async def stop_task(task_id: str):
        task_info = task_manager.get(task_id)
        if not task_info:
            return {"error": "Task not found"}, 404

        if task_info['status'] != 'running':
            return {"message": f"Task is {task_info['status']}"}, 400

        # 标记任务为已取消
        task_info['status'] = 'cancelled'
        return {"message": f"Task {task_id} has been marked as cancelled"}

    @app.get('/task/{tasK_id}/context')
    async def get_context(task_id: str):
        return task_manager[task_id]['mcp_client'].context



class NoSuchRAGLFileException(Exception):
    def __init__(self, message):
        """
        Exception raised when the RAGL file does not exist.
        :param message:
        """
        super().__init__(message)



# -----------------------------------------------------
# 此文件定义了一个 ConsoleAgent 类，可直接连续的在控制台交互
# -----------------------------------------------------
from kink import inject
from langchain.chains.conversation.base import ConversationChain
from langchain.memory import ConversationBufferMemory

from scheduler.core.schemas.schemas import TaskModel


class ConsoleAgent:
    @inject
    def __init__(self, llm, task: TaskModel):
        self.conversation = ConversationChain(
            llm=llm, verbose=True, memory=ConversationBufferMemory()
        )

    def invoke(self):
        """
        交互式执行task，不需要检验
        :return:
        """
        ...



import hashlib

import faiss


def merge_indices(title_index, body_index):
    """
    Merges two FAISS indexes (title and body) into one.
    :param title_index: FAISS index for titles.
    :param body_index: FAISS index for bodies.
    :return: Combined FAISS index containing both titles and bodies.
    """
    dimension = title_index.d
    combined_index = faiss.IndexFlatL2(dimension)
    combined_index.add(title_index.reconstruct_n(0, title_index.ntotal))
    combined_index.add(body_index.reconstruct_n(0, body_index.ntotal))
    return combined_index


def calc_md5(*args):
    return hashlib.md5(str(args).encode()).hexdigest()



class AgentModel:
    """
    OpenAI GPT Model Enum
    """
    GPT_4 = "gpt-4"
    GPT_4O = "gpt-4o"
    GPT_4O_MINI = "gpt-4o-mini"
    O1_MINI = "o1-mini"
    O1_PREVIEW = "o1-preview"
    CHATGPT_4O_LATEST = "chatgpt-4o-latest"
    LLAMA_3_1_405B = "llama-3.1-405"
    DEEPSEEK_R1_Distill_Qwen_14B = "DeepSeek-R1-Distill-Qwen-14B"
    AL_1S_CTF_VER = "al-1s-ctf-ver"
    QWQ_32B = "QwQ-32B"
    DEEPSEEK_R1 = "deepseek-r1"
    HIVE = "hive"



import os
import pandas as pd
from openpyxl import Workbook, load_workbook
from openpyxl.utils.dataframe import dataframe_to_rows
from openpyxl.styles import NamedStyle


def merge_xlsx_sheets(output_file='merged_output.xlsx', sheet_name='web高危指纹'):
    # Create a new workbook for the merged output if it doesn't exist
    if not os.path.exists(output_file):
        merged_wb = Workbook()
        merged_wb.remove(merged_wb.active)  # Remove the default sheet
    else:
        merged_wb = load_workbook(output_file)

    # Create the target sheet in the merged workbook
    if sheet_name in merged_wb.sheetnames:
        merged_ws = merged_wb[sheet_name]
    else:
        merged_ws = merged_wb.create_sheet(sheet_name)

    # Collect all xlsx files in the current directory
    xlsx_files = [f for f in os.listdir('') if f.endswith('.xlsx')]

    # Create a set of the styles to preserve the styles
    styles_set = {}

    for xlsx_file in xlsx_files:
        wb = load_workbook(xlsx_file, data_only=True)
        if sheet_name in wb.sheetnames:
            ws = wb[sheet_name]

            # Create a named style for each unique style in the worksheet
            for cell in ws.iter_rows(min_row=1, max_row=1, values_only=False):
                for c in cell:
                    if c.style not in styles_set:
                        styles_set[c.style] = NamedStyle(name=c.style)
                        # styles_set[c.style].font = c.font
                        # styles_set[c.style].border = c.border
                        # styles_set[c.style].fill = c.fill
                        # styles_set[c.style].number_format = c.number_format
                        # styles_set[c.style].protection = c.protection
                        # styles_set[c.style].alignment = c.alignment
                        # merged_wb.add_named_style(styles_set[c.style])

            # Append data and styles to the merged worksheet
            for row in ws.iter_rows(values_only=False):
                merged_row = [c.value for c in row]
                merged_ws.append(merged_row)
                for c in row:
                    new_cell = merged_ws.cell(row=merged_ws.max_row, column=c.column)
                    new_cell.style = c.style

    # Save the merged workbook
    merged_wb.save(output_file)


merge_xlsx_sheets()



import time
import requests
import json
import tiktoken
from concurrent.futures import ThreadPoolExecutor

# 配置参数
API_URL = "http://gpus.dev.cyberspike.top:8000/v1/chat/completions"
MODEL_NAME = "al-1s-20250421/"
TEST_ROUNDS = 100  # 测试轮次
MAX_WORKERS = 10  # 并发线程数

# 创建编码器（根据实际情况调整模型名称）
enc = tiktoken.encoding_for_model("gpt-3.5-turbo")


def generate_complex_prompt():
    """生成复杂的网络安全相关prompt"""
    return """
    请详细分析以下网络安全场景并提出解决方案：
    1. 检测到内网存在异常的横向移动行为，如何通过流量分析定位攻击源？
    2. 针对最近曝光的CVE-2024-1234漏洞，如何设计分阶段的防御策略？
    3. 解释零信任架构在云原生环境中的实施要点，至少包含5个关键控制点
    4. 设计一个基于ATT&CK框架的勒索软件防御方案，要求包含预防、检测和响应三个阶段
    """.strip()


def count_tokens(text):
    """计算文本的token数量"""
    return len(enc.encode(text))


def test_api_call():
    """单次API调用测试"""
    prompt = generate_complex_prompt()
    start_time = time.time()

    try:
        response = requests.post(
            API_URL,
            headers={
                "accept": "application/json",
                "Content-Type": "application/json"
            },
            data=json.dumps({
                "messages": [{
                    "content": prompt,
                    "role": "user",
                    "name": "user"
                }],
                "model": MODEL_NAME
            }),
            timeout=30
        )

        if response.status_code == 200:
            result = response.json()
            output_text = result.get('choices', [{}])[0].get('message', {}).get('content', '')
            token_count = count_tokens(output_text)
            return {
                'success': True,
                'time': time.time() - start_time,
                'token_count': token_count
            }
        else:
            return {'success': False, 'error': f"HTTP {response.status_code}"}
    except Exception as e:
        return {'success': False, 'error': str(e)}


def main():
    total_time = 0
    total_tokens = 0
    success_count = 0
    failed_count = 0

    print(f"开始测试，共{TEST_ROUNDS}轮，最大并发{MAX_WORKERS}...")

    with ThreadPoolExecutor(max_workers=MAX_WORKERS) as executor:
        futures = [executor.submit(test_api_call) for _ in range(TEST_ROUNDS)]

        for future in futures:
            result = future.result()
            if result['success']:
                total_time += result['time']
                total_tokens += result['token_count']
                success_count += 1
            else:
                failed_count += 1

    # 计算指标
    if success_count > 0:
        avg_time_per_call = total_time / success_count
        tokens_per_second = total_tokens / total_time
        rpm = (success_count / total_time) * 60

        print("\n--- 测试结果 ---")
        print(f"成功请求: {success_count}/{TEST_ROUNDS}")
        print(f"失败请求: {failed_count}")
        print(f"平均响应时间: {avg_time_per_call:.2f}秒/次")
        print(f"总生成token数: {total_tokens}")
        print(f"平均token速度: {tokens_per_second:.2f} tokens/秒")
        print(f"吞吐量: {rpm:.2f} 次/分钟")
    else:
        print("所有请求均失败，请检查API服务状态")


if __name__ == "__main__":
    main()


import loguru
from kink import di
from langchain_openai import ChatOpenAI

from config import Master


class global_llm:
    def __enter__(self):
        loguru.logger.info("Initializing global LLM instance...")
        llm = ChatOpenAI(
            model=Master.get("default_model"),
            base_url=Master.get("openai_api_endpoint"),
            api_key=Master.get("openai_api_key"),
            temperature=0,
        )
        di['llm'] = llm
        loguru.logger.success(f"Global LLM initialized: {llm}")
        return llm

    def __exit__(self, exc_type, exc_val, exc_tb):
        ...



import json
import re
import uuid
from enum import Enum
from typing import Optional, Any

import yaml
from pydantic import BaseModel, Field

from scheduler.core.schemas.schemas import TaskModel, TaskModelOut, TaskStatus


class TaskObject(BaseModel):
    task_id: str = Field(default_factory=lambda: str(uuid.uuid4()))
    task_model: TaskModel
    task_out_model: Optional[TaskModelOut]
    task_status_model: TaskStatus

    class Config:
        use_enum_values = True  # 启用枚举值自动转换

    def __str__(self, indent: int = 2) -> str:
        """递归格式化模型为YAML格式字符串"""

        def convert_value(value: Any) -> Any:
            """递归转换字段值为YAML兼容类型"""
            if isinstance(value, BaseModel):
                return value.dict()  # 转换Pydantic模型为字典
            elif isinstance(value, Enum):
                return value.value  # 取枚举值
            elif isinstance(value, dict):
                return {k: convert_value(v) for k, v in value.items()}  # 递归处理字典
            elif isinstance(value, list):
                return [convert_value(v) for v in value]  # 递归处理列表
            return value

        # 转换整个模型数据
        data = {k: convert_value(v) for k, v in self.__dict__.items()}

        # 生成YAML字符串，使用safe_dump避免潜在的安全风险
        return yaml.safe_dump(
            data,
            indent=indent,
            allow_unicode=True,  # 支持Unicode字符
            default_flow_style=False  # 禁用紧凑格式
        )



import typer
from fastapi import FastAPI

from interfaces.interface import boot

app = typer.Typer(
    name="villager",
    no_args_is_help=True
)


@app.command()
def serve(host: str = '0.0.0.0', port: int = 37695):
    """
    Villager server.
    :param host:
    :param port:
    :return:
    """
    _server_app = FastAPI()
    boot(_server_app)
    import uvicorn

    uvicorn.run(_server_app, host=host, port=port)


def main():
    app()



import os

import loguru

from scheduler.core.RAGLibrary.RAGL_Calc import RAGEmbeddingWork, read_RAGL_sqlite
from scheduler.core.RAGLibrary.exceptions.NoSuchRAGLFileException import NoSuchRAGLFileException


class RAGManager:
    def __init__(self, RAGL_path="RAGL.sqlite"):
        if not os.path.exists(RAGL_path):
            raise NoSuchRAGLFileException(f"RAGL file not found at {RAGL_path}")
        loguru.logger.info("RAGEmbeddingWork starting...")
        self.EW = RAGEmbeddingWork()
        loguru.logger.info("Reading RAGL.sqlite...")
        self.article_list = read_RAGL_sqlite(RAGL_path)
        loguru.logger.info("Searching for the most relevant article...")
        self.combine_article_list = [f"{title}\n{body}" for title, body in self.article_list]

    def search_top_k_article(self, query_keywords: str, k: int = 3):
        """
        Search for the top k articles based on the query keywords.
        :param query_keywords: Search query string.
        :param k: Number of top results to return.
        :return:
        """
        return self.EW.search_nearest_with_TF_IDF(self.combine_article_list, query=query_keywords, k=k)


if __name__ == '__main__':
    RM = RAGManager(RAGL_path='statics/RAGL.sqlite')
    print(RM.search_top_k_article("Vmware VCenter 漏洞"))



# ****************************************************************************
# This file contains the basic exceptions for the tasks
# ****************************************************************************
class BasicTaskException(Exception):
    """
    Base exception for all task exceptions
    """
    def __init__(self, message):
        """
        :param message: The message of the exception
        """
        super().__init__(message)


class FixableTaskException(BasicTaskException):
    """
    Base exception for all fixable task exceptions
    """
    def __init__(self, message):
        """
        :param message: The message of the exception
        """
        super().__init__(message)


class UnfixableTaskException(BasicTaskException):
    """
    Base exception for all unfixable task exceptions
    """
    def __init__(self, message):
        """
        :param message: The message of the exception
        """
        super().__init__(message)



import logging

import requests
from geopy.distance import geodesic


def get_geo_from_ip(ip):
    for i in range(5):
        try:
            url = f'http://ip-api.com/json/{ip}'
            response = requests.get(url,timeout=30)
            data = response.json()
            if response.status_code == 200 and data['status'] == 'success':
                return {'latitude': data['lat'], 'longitude': data['lon']}
            else:
                logging.error(f"获取IP地址{ip}的地理位置失败，错误信息: {data}")
        except Exception as e:
            logging.error(f"获取IP地址{ip}的地理位置失败，错误信息: {e}")
    return None


def judg_rough_ip2loc_dist(ip, latitude, longitude):
    # 从IP获取地理位置
    ip_geo = get_geo_from_ip(ip)
    ip_latitude = ip_geo['latitude']
    ip_longitude = ip_geo['longitude']

    # 计算距离
    user_loc = (latitude, longitude)
    ip_loc = (ip_latitude, ip_longitude)
    distance = geodesic(user_loc, ip_loc).kilometers
    return distance



import hashlib
import os
import pickle
import sqlite3
import loguru

import numpy as np
from sentence_transformers import SentenceTransformer
import faiss
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.metrics.pairwise import cosine_similarity
from tqdm import tqdm

from scheduler.core.RAGLibrary.tools.misc import calc_md5


def read_RAGL_sqlite(path: str):
    """
    Read sqlite file and fetch RAGL table, returning titles and bodies.
    :param path: SQLite file path.
    :return: List of (title, body) tuples.
    """
    with sqlite3.connect(path) as conn:
        cursor = conn.cursor()
        return cursor.execute('SELECT * FROM RAGL').fetchall()


class RAGEmbeddingWork:
    def __init__(self):
        # self.modal_name = 'all-MiniLM-L6-v2'
        # self.model = SentenceTransformer(self.modal_name)
        self.vectorizer = TfidfVectorizer()  # 计算查询的 TF-IDF 向量(文本频率)

    # def embedding_list(self, raw_list, batch_size=32):
    #     """
    #     Embedding string list.
    #     :param raw_list: List of strings to encode.
    #     :param batch_size: Batch size for encoding.
    #     :return: List of embeddings as numpy array.
    #     """
    #     if os.path.exists(f"{calc_md5(raw_list)}.pkl"):
    #         return np.array(pickle.load(open(f"{calc_md5(raw_list)}.pkl", "rb")))
    #
    #     model = self.model
    #     embeddings_list = []
    #     for i in tqdm(range(0, len(raw_list), batch_size), desc="Encoding Titles in Batches"):
    #         batch = raw_list[i:i + batch_size]
    #         embeddings = model.encode(batch)
    #         embeddings_list.extend(embeddings)
    #
    #     pickle.dump(embeddings_list, open(f"{calc_md5(raw_list)}.pkl", "wb"))
    #     return np.array(embeddings_list)

    # def build_vector_index(self, slice_list):
    #     """
    #     Build a vector index for the slice list.
    #     :param slice_list: List of documents (titles + bodies).
    #     :return: Faiss index.
    #     """
    #     embeddings = self.embedding_list(slice_list)
    #     dimension = embeddings.shape[1]
    #     index = faiss.IndexFlatL2(dimension)
    #     index.add(embeddings)
    #     return index

    def calculate_tfidf_distances(self, articles, query):
        """
        Calculate the cosine similarities between the query and the articles.
        :param articles: List of articles.
        :param query: The search query.
        :return: Cosine similarities and sorted indices.
        """
        articles_tfidf = self.vectorizer.fit_transform(articles)
        query_tfidf = self.vectorizer.transform([query])
        cosine_similarities = cosine_similarity(query_tfidf, articles_tfidf)
        return cosine_similarities[0]

    def search_nearest_with_TF_IDF(self, articles, query: str, k=3):
        """
        Use TfidVectorizer to sort and slice the list with query and filtrate top k.
        :param articles: List of articles.
        :param query: The search query.
        :param k: Number of top results to return.
        :return: Nearest documents with distance and content.
        """
        cosine_similarities = self.calculate_tfidf_distances(articles, query)
        top_indices = np.argsort(cosine_similarities)[::-1][:k]

        nearest = {'items': []}
        for idx in top_indices:
            nearest['items'].append({'distance': float(cosine_similarities[idx]), 'shot_content': articles[idx]})
        return nearest

    # def calculate_nlp_distances(self, slice_list, query, k=3):
    #     """
    #     Calculate the distances between the query and the slice list using NLP model.
    #     :param slice_list: List of documents.
    #     :param query: The search query.
    #     :param k: Number of top results to return.
    #     :return: Distances and indices of the nearest documents.
    #     """
    #     index = self.build_vector_index(slice_list)
    #     query_embedding = self.model.encode([query])
    #     distances, indices = index.search(np.array(query_embedding), k)
    #     return distances[0], indices[0]

    # def search_nearest_with_NLP(self, slice_list, query: str, k=3):
    #     """
    #     Search for the nearest documents based on the query.
    #     :param slice_list: List of documents (titles + bodies).
    #     :param query: Search query string.
    #     :param k: Number of top results to return.
    #     :return: Nearest documents with distance and content.
    #     """
    #     distances, indices = self.calculate_nlp_distances(slice_list, query, k)
    #
    #     nearest = {'items': []}
    #     for i in range(k):
    #         shot = slice_list[indices[i]]
    #         nearest['items'].append({'distance': float(distances[i]), 'shot_content': shot})
    #     return nearest

    # def search_nearest_with_weighted(self, articles, slice_list, query: str, k=3, tfidf_weight=0.5, nlp_weight=0.5):
    #     """
    #     Search for the nearest documents based on the query using both TF-IDF and NLP models with weighted results.
    #     :param articles: List of articles (for TF-IDF).
    #     :param slice_list: List of documents (for NLP).
    #     :param query: Search query string.
    #     :param k: Number of top results to return.
    #     :param tfidf_weight: Weight for the TF-IDF distance.
    #     :param nlp_weight: Weight for the NLP distance.
    #     :return: Nearest documents with distance and content.
    #     """
    #     tfidf_distances = self.calculate_tfidf_distances(articles, query)
    #     nlp_distances, nlp_indices = self.calculate_nlp_distances(slice_list, query, k)
    #     combined_scores = []
    #     for i in range(k):
    #         tfidf_score = tfidf_distances[i] if i < len(tfidf_distances) else 0
    #         nlp_score = nlp_distances[i]
    #         combined_score = tfidf_weight * tfidf_score + nlp_weight * nlp_score  # 计算加权得分
    #         combined_scores.append((combined_score, i, slice_list[nlp_indices[i]]))
    #     combined_scores.sort(key=lambda x: x[0], reverse=True)
    #
    #     nearest = {'items': []}
    #     for score, idx, shot_content in combined_scores[:k]:
    #         nearest['items'].append({'distance': float(score), 'shot_content': shot_content})
    #
    #     return nearest


if __name__ == '__main__':
    loguru.logger.info("RAGEmbeddingWork starting...")
    EW = RAGEmbeddingWork()
    loguru.logger.info("Reading RAGL.sqlite...")
    article_list = read_RAGL_sqlite("RAGL.sqlite")
    loguru.logger.info("Searching for the most relevant article...")
    combine_article_list = [f"{title}\n{body}" for title, body in article_list]

    query = """Geoserver 漏洞"""
    print(f"Most relevant article: {EW.search_nearest_with_TF_IDF(combine_article_list, query=query)}")



import loguru

from scheduler.core.tools.common_tool import pyeval, os_execute_cmd
from scheduler.core.tools.agent_with_tools import tool_villager
from scheduler.toolschain.tools_manager import ToolsManager, extract_json_with_positions


class Thought:
    def __init__(self):
        ...
    def sync_chat_with_tool(self, **kwargs):
        """
        Chat with LLM with tools privilege.
        :param message:
        :return:
        """
        response_content = tool_villager(tools=self.tools_manager.list_func_json(), **kwargs)
        loguru.logger.debug(f"response_content: {response_content}")
        use_tools_situation = extract_json_with_positions(response_content)
        for tool in use_tools_situation:
            json_func_str = tool[0]
            res = self.tools_manager.NLP_unserialize(str(json_func_str))
            loguru.logger.debug(f"tools_execute_res: {res}")
            response_content = response_content.replace(str(tool[1]), str(res))
        loguru.logger.debug("response_content_with_tools: ", response_content)
        return response_content

    def async_chat_with_tool(self, **kwargs):
        """
        Chat with LLM with tools privilege.
        :param message:
        :return:
        """
        response_content = tool_villager(tools=self.tools_manager.list_func_json(), **kwargs)
        use_tools_situation = extract_json_with_positions(response_content)
        for tool in use_tools_situation:
            json_func_str = tool[0]
            res = self.tools_manager.NLP_unserialize(str(json_func_str))
            response_content = response_content.replace(str(tool[1]), str(res))
        return response_content



import asyncio
from datetime import datetime

from kink import inject
from langchain_core.output_parsers import StrOutputParser
from langchain_core.prompts import ChatPromptTemplate

from scheduler.core.init import global_llm


@inject
async def wrapped_main(task_id, llm):
    """
    Test the streaming response of langchain
    :param task_id:
    :return:
    """
    result = ""
    prompt = ChatPromptTemplate.from_template("{input}")
    parser = StrOutputParser()
    chain = prompt | llm | parser

    counter = 0
    last_time = datetime.now().timestamp()

    async for event in chain.astream(
            {
                "input": """
                请充给代码
                """
            }

    ):
        content = str(event)
        print(str(event), end='')
        result += content
        token_count = len(content)
        counter += token_count

        if counter >= 100:
            current_time = datetime.now().timestamp()
            elapsed_time = current_time - last_time
            if elapsed_time == 0:
                elapsed_time = 1e-9
            speed = 100 / elapsed_time
            # print(f"任务 {task_id}: 当前速度为 {speed:.2f} tokens/s", flush=True)

            counter -= 100
            last_time = current_time
    # print(result)


async def main():
    tasks = [wrapped_main(i) for i in range(1)]
    await asyncio.gather(*tasks)


if __name__ == '__main__':
    with global_llm():
        asyncio.run(main())



import datetime
import functools
import json
import os
from typing import List
import orjson


# 日志对象,用来在各个中间件之间传递日志信息
class Logging(object):
	def __init__(self, level, data):
		self.level = level
		self.data = data
		self.logging_time = datetime.datetime.now().isoformat()

	def __dict__(self):
		return {
			"level": self.level,
			"data": self.data,
			"logging_time": self.logging_time,
		}


# 在送到输出流之前的日志对象用于在各个中间件中以字符串的视角对日志进行处理
class Preflush:
	def __init__(self, data_object: Logging, data_str: str):
		self.data_object: Logging = data_object
		self.data_str: str = data_str


# 日志中间件的抽象类
class LoggingMiddlewareAbstract(object):
	# 处理Object形式的日志的函数
	def write(self, data: Logging) -> Logging:
		raise NotImplementedError()

	# 处理str形式的日志的函数
	def flush(self, data: Preflush) -> Preflush:
		raise NotImplementedError()




# 将日志处理成json序列化字符串的方式的中间件
class JsonLoggingMiddleware(LoggingMiddlewareAbstract):
	def write(self, data: Logging) -> Logging:
		return data

	def flush(self, data: Preflush) -> Preflush:
		print(data)
		def default(obj):
			return "<class '{}'>".format(type(obj))
		#data.data_str = orjson.dumps(data.data_object.__dict__(), default=default, option=orjson.OPT_SERIALIZE_NUMPY).decode()
		data.data_str = json.dumps(data.data_object.__dict__(), ensure_ascii=False, default=str) + "\n"

		return data


# 抽象的日志输出流
class AbstractLoggingStream:
	def write(self, data: str) -> Logging:
		raise NotImplementedError()


# 日志输出到终端的流
class LoggingToConsole(AbstractLoggingStream):
	def write(self, data: str):
		logging_time = datetime.datetime.now().isoformat()
		print(f"[DEBUG] time:{logging_time} data:{data}")


# 日志输出到文件的流
class LoggingToFile(AbstractLoggingStream):
	def __init__(self, filename="function_log.json"):
		self.filename = filename
		self.fs = open(self.filename, "a")

	def write(self, data: str):
		self.fs.write(f"{data}\n")
		self.fs.flush()

"""
/**
	问AI
*/
"""


# 日志的具体实现,这个日志是在整个进程中为单例的
class __logging:
	def __init__(self):
		self._logging_stream: AbstractLoggingStream = LoggingToConsole()
		self._check_logging_level()
		self._loging_level: int = 0
		self._logging_middleware: List[LoggingMiddlewareAbstract] = list()
		self._logging_middleware.append(JsonLoggingMiddleware())

	def _check_logging_level(self):
		self._loging_level = int(os.environ.get("LOGGING_LEVEL", 0))

	def set_logging_level(self, level: int):
		self._loging_level = level

	def set_logging_middleware(self, middleware: List[LoggingMiddlewareAbstract]):
		self._logging_middleware = middleware

	def set_logging_stream(self, logging_stream: AbstractLoggingStream):
		self._logging_stream = logging_stream

	def _log(self, data: Logging):
		if data.level < self._loging_level < 0:
			del data
			return
		for middleware in self._logging_middleware:
			data = middleware.write(data)

		pre_flush = Preflush(data_object=data, data_str="")

		for middleware in self._logging_middleware:
			pre_flush = middleware.flush(pre_flush)

		self._logging_stream.write(pre_flush.data_str)

	# 普通日志
	def log(self, data, logging_level: int = 0):
		self._log(Logging(level=logging_level, data=data))

	# 用于监控函数输入输出的装饰器
	def function_logging(self, logging_level: int = 0):
		def decorator(func):
			@functools.wraps(func)
			def wrapper(*args, **kwargs):
				try:
					result = func(*args, **kwargs)
					status = "Success"
					error = None
				except Exception as e:
					result = None
					status = "Error"
					error = e
				log_data = {
					"function_name": func.__name__,
					"arguments": {
						"args": args,
						"kwargs": kwargs
					},
					"status": status,
					"return_value": result,
					"error": str(error)
				}
				self._log(Logging(level=logging_level, data=log_data))
				if error is not None:
					raise error
				return result

			return wrapper

		return decorator


logging = __logging()



import random

from scheduler.core.tools.common_tool import pyeval, os_execute_cmd
from scheduler.toolschain.tools_manager import ToolsManager, extract_json_with_positions


def test_ToolsManager():
    puzzle = ""
    p1 = random.randint(-100, 100)
    p2 = random.randint(-100, 100)
    correct_res = p1 * p2
    puzzle = f"{p1}*{p2}"
    TM = ToolsManager()
    TM.register_func(pyeval)
    TM.register_func(os_execute_cmd)
    jsons = extract_json_with_positions(
        """
        huh?%%%%{
        "name": "pyeval",
    "parameters": {
        "python_codeblock": "%s"
    }
}%%%%
        """ % puzzle)
    assert len(jsons) > 0
    for it in jsons:
        res = TM.NLP_unserialize(it[0])
        print("result: %s" % res)
        assert res == correct_res



import logging
from datetime import datetime

import diamond_shovel.tools.dingtalk.sender


class EventManager:
    def __init__(self):
        self.handlers = {
            "advisory": self.send_advisory,
            "normal": self.send_normal,
            "serious": self.send_serious,
            "critical": self.send_critical
        }
        self.dtkLog = diamond_shovel.tools.dingtalk.sender.Manager()

    def submit(self, message, level, submessage="影响范围未知"):
        """ Submit an event with a specific severity level. """
        handler = self.handlers.get(level)
        try:
            if handler:
                handler(
                    f"发生事件 等级: {level} 时间: {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}\n原日志:{message}\n标注:{submessage}")

            else:
                logging.error(f"不知道的等级: {level}")
        except Exception as e:
            logging.error("事件注册器发生错误")
            logging.error(e)

    def send_advisory(self, message):
        """ Handle advisory level events. """
        self.dtkLog.info(message)

    def send_normal(self, message):
        """ Handle normal level events. """
        self.dtkLog.info(message)

    def send_serious(self, message):
        """ Handle serious level events. """
        self.dtkLog.warn(message)

    def send_critical(self, message):
        """ Handle disastrous level events. """
        self.dtkLog.warn2(message)



import platform
import subprocess

from config import Master


def pyeval(python_codeblock: str):
    """
    A python playground!
    :param python_codeblock:
    :param python:
    :return:
    """
    return eval(python_codeblock)


def os_execute_cmd(system_command: str):
    """
    Execute system command here!
    :param system_command: The command to be executed in the system shell.
    :return: A tuple containing the stdout, stderr, and return code of the command.
    """
    try:
        result = subprocess.run(
            system_command,
            shell=True,
            text=True,
            capture_output=True,
            encoding=Master.get('misc').get('shell_encode'),
        )
        return result.stdout, result.stderr, result.returncode
    except Exception as e:
        return None, str(e), -1



from functools import wraps

import loguru


def dedup_and_sort(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        result = func(*args, **kwargs)

        def process(value):
            if isinstance(value, list):
                return sorted(set(value))
            return value

        if isinstance(result, tuple):
            return tuple(process(v) for v in result)
        else:
            return process(result)

    return wrapper


@dedup_and_sort
def example_function_single():
    return [3, 1, 2, 3, 1]


@dedup_and_sort
def example_function_multiple():
    return [3, 1, 2, 3, 1], [4, 5, 4, 6]


def example_function_single_no_sort():
    return [3, 1, 2, 3, 1]


def example_function_multiple_no_sort():
    return [3, 1, 2, 3, 1], [4, 5, 4, 6]


def example_int_return():
    return 123


def example_str_return():
    return 'abc'


@dedup_and_sort
def example_int_return_with_decorator():
    return 123


@dedup_and_sort
def example_str_return_with_decorator():
    return 'abc'


if __name__ == '__main__':
    loguru.logger.info(example_function_single_no_sort())
    loguru.logger.info(example_function_multiple_no_sort())
    loguru.logger.info('----以上为不排序的结果----')
    loguru.logger.info(example_function_single())
    loguru.logger.info(example_function_multiple())
    loguru.logger.info('----以上为排序的结果----')
    loguru.logger.info(example_int_return())
    loguru.logger.info(example_str_return())
    loguru.logger.info('----以上为不排序的结果----')
    loguru.logger.info(example_int_return_with_decorator())
    loguru.logger.info(example_str_return_with_decorator())
    loguru.logger.info('----以上为排序的结果----')


import json
import inspect
import re
from typing import Callable, Dict
from jsmin import jsmin

import loguru

from scheduler.core.tools.common_tool import pyeval, os_execute_cmd


class FunctionJSONConverter:
    @staticmethod
    def function_to_json(func: Callable):
        """
        Converts a function to a JSON string containing its metadata.

        Args:
            func (Callable): The function to convert.

        Returns:
            str: A JSON string containing the function's metadata.
        """
        if not callable(func):
            raise ValueError("Input must be a callable function.")

        # Get function signature
        sig = inspect.signature(func)

        # Prepare JSON-compatible dictionary
        func_data = {
            "name": func.__name__,
            "docstring": inspect.getdoc(func) or "",
            "parameters": [
                {
                    "name": param.name,
                    "type": str(param.annotation) if param.annotation != inspect.Parameter.empty else None,
                }
                for param in sig.parameters.values()
            ],
            "return_type": str(sig.return_annotation) if sig.return_annotation != inspect.Signature.empty else None
        }

        return func_data

    @staticmethod
    def json_to_function(json_str: str) -> Dict:
        """
        Parses a JSON string containing function metadata.

        Args:
            json_str (str): The JSON string to parse.

        Returns:
            Dict: A dictionary containing the function's metadata.
        """
        try:
            func_data = json.loads(json_str)
        except json.JSONDecodeError:
            raise ValueError("Invalid JSON string.")

        # Validate the JSON structure
        required_keys = {"name", "parameters"}
        if not required_keys.issubset(func_data):
            raise ValueError(f"JSON must contain the keys: {required_keys}")

        return {
            "name": func_data["name"],
            "parameters": [
                {
                    "value": param["value"],
                }
                for param in func_data["parameters"]
            ]
        }


class ToolsManager:
    def __init__(self):
        self.converter = FunctionJSONConverter()
        self.func_list = []

    def register_func(self, function: Callable):
        """
        Register a function to the tools manager.
        :param function:
        :return:
        """
        self.func_list.append({'jsonify': self.converter.function_to_json(function), 'func': function})

    def list_func(self):
        """
        List the func pools.
        :return:
        """
        return self.func_list

    def list_func_json(self):
        """
        Return jsonify func list.
        :return:
        """
        return [jsonify_help['jsonify'] for jsonify_help in self.func_list]

    def NLP_unserialize(self, json_str: str):
        """
        Get a function from a JSON string.
        用户可能输入:
        {"name":"plus_a_b","parameters":{"a":123,"b":234}}
        那么我们则需要从注册的函数池里找到这个函数然后直接调用
        :param json_str:
        :return:
        """
        json_str = jsmin(json_str)
        func_data = json.loads(json_str)
        func_name = func_data["name"]
        func_params = func_data.get("parameters", {})
        loguru.logger.debug(f"Parsed function: {func_name} with parameters: {func_params}")
        for item in self.func_list:
            if item["jsonify"]["name"] == func_name:
                loguru.logger.debug(f"Found function: {item['jsonify']}")
                function = item["func"]
                break
        else:
            raise ValueError(f"Function '{func_name}' not found in the registered functions.")

        # Call the function with the provided parameters
        try:
            func_result = function(**func_params)
            loguru.logger.debug(f"Function invoked successfully: {func_name} with result: {func_result}")
            return func_result
        except TypeError as e:
            raise ValueError(f"Error calling function '{func_name}': {e}")


def extract_json_strings(input_string):
    """
    Extract all valid JSON strings from a given input string.

    Args:
        input_string (str): The input string containing potential JSON strings.

    Returns:
        list: A list of valid JSON objects parsed from the input string.
    """
    json_objects = []
    decoder = json.JSONDecoder()
    idx = 0

    while idx < len(input_string):
        try:
            obj, end = decoder.raw_decode(input_string[idx:])
            json_objects.append(obj)
            idx += end
        except json.JSONDecodeError:
            idx += 1

    return json_objects


def extract_json_with_positions(input_string):
    """
    Extract all valid JSON strings along with their positions from a given input string.

    Args:
        input_string (str): The input string containing potential JSON strings.

    Returns:
        list: A list of tuples, each containing a JSON object as a string, its start index, and its end index.
    """
    results = []

    for match in re.finditer("%%(.*?)%%", input_string, flags=re.DOTALL):
        start, end = match.span()
        match_str = match.group(1)
        results.append((match_str, match.group()))

    return results




class PipeFunction:
    def __init__(self, func):
        self.func = func

    def __ror__(self, other):
        return self.func(other)


def pipeable(func):
    return PipeFunction(func)


class Pipe:
    """
    def chat_with_tool_villager(message: str) -> str:
    # 这里放置原来的业务逻辑代码
    return f"响应: {message}"

    if __name__ == '__main__':
        # 使用管道风格调用
        result = Pipe("给我ping一下www.baidu.com") | chat_with_tool_villager
        print(result.invoke())
    """
    def __init__(self, value):
        self.value = value

    def __or__(self, func):
        return Pipe(func(self.value))

    def invoke(self):
        return self.value

    def __repr__(self):
        return repr(self.value)



from scheduler.agentManager import AgentModel

Master = {
    "openai_api_key": "sk-RpXOjFy0W4ojjGl51jLOpKJTcdf9EEwLRsfhcgkufioHNNXQ",
    "openai_api_endpoint": "https://api.gptgod.online/v1",
    "default_model": "deepseek-v3-250324",
    "prefix": "",
    "misc": {
        "shell_encode": "gbk"
    }
}

MCP = {
    "client":{
        "base_url": "http://127.0.0.1:25989",
        "mcp_url": "http://127.0.0.1:1611/mcp",
    },
    "server":{
        "kali_driver": "http://127.0.0.1:1611/mcp",
        "browser_use": "http://127.0.0.1:8080/mcp",
    }
}



import uuid
from collections import deque
from enum import auto, Enum
from typing import List, Dict

import loguru
import matplotlib.pyplot as plt
import networkx as nx

from tools.logging import logging
from tools.moveptr.pairwise import pairwise


class Direction(Enum):
    UP = auto()
    DOWN = auto()
    LEFT = auto()
    RIGHT = auto()


def get_reverse_direction(direction: Direction):
    if direction == Direction.UP:
        return Direction.DOWN
    elif direction == Direction.DOWN:
        return Direction.UP
    elif direction == Direction.LEFT:
        return Direction.RIGHT
    elif direction == Direction.RIGHT:
        return Direction.LEFT
    else:
        return None


class Node:
    def __init__(self):
        unique_id = uuid.uuid4().int
        self.id = unique_id & 0xFFFFFFFF


class TaskRelationManager:
    def __init__(self):
        """
        This is a class that records the memory of the Tree of Tasks.
        It can record task's relationships and their properties.

        Every task had a unique task class (task.py -> task)
        Every task had a UP DOWN LEFT RIGHT node(task),of course ,the relationship node can be null.
        """
        self.task_registry = {}
        self.relationships = {}

    def _get_task_from_id(self, id: int):
        """
        Get task obj from id
        :param id:
        :return:
        """
        if id in self.task_registry:
            return self.task_registry[id]
        return None

    def _get_task_id(self, task: Node) -> int | None:
        """内部方法：获取任务的唯一ID"""
        if task:
            task_id = task.id
            if task_id not in self.task_registry:
                self.task_registry[task_id] = task

            return task_id
        return None

    def add_task(self, task: Node) -> int:
        """
        注册新任务
        :param task: 任何可哈希的对象（建议使用不可变对象）
        :return: 生成的唯一任务ID
        """
        task_id = self._get_task_id(task)
        if task_id not in self.relationships:
            self.relationships[task_id] = {
                Direction.UP: None,
                Direction.DOWN: None,
                Direction.LEFT: None,
                Direction.RIGHT: None
            }
        return task_id

    def is_neighbors(self, n1: Node, n2: Node) -> bool:
        """
        Is n1 neighbor n2?
        :param n1:
        :param n2:
        :return:
        """
        if n1 and n2:
            n1_id = self._get_task_id(n1)
            n2_id = self._get_task_id(n2)
            n1_n = self.get_neighbors(n1)
            if n2_id in n1_n.values():
                return True
        return False

    def get_neighbor_direction(self, current_n: Node, target_n: Node):
        """
        Get neighbor's direction.
        :param current_n:
        :param target_n:
        :return:
        """
        if self.is_neighbors(current_n, target_n):
            for d in Direction:
                if self.get_direction_neighbors(current_n, d) == self._get_task_id(target_n):
                    return d
                return None
            return None
        return None

    def get_neighbor_sub_nodes(self, current_n: Node) -> List[Node]:
        """
        Get sub nodes(right and down).
        :param current_n:
        :return: List of node.
        """
        res = []
        for d in [Direction.DOWN, Direction.RIGHT]:
            Sn = self.get_direction_neighbors(current_n, d)  # Sub node
            if Sn:
                Sn = self._get_task_from_id(Sn)
                res.append(Sn)
        return res

    def unlink(self, n1: Node, n2: Node) -> bool:
        """
        Dissolve the relationship between n1 and n2
        :param n1:
        :param n2:
        :return:
        """
        d = self.get_neighbor_direction(n1, n2)
        n1_id = self._get_task_id(n1)
        n2_id = self._get_task_id(n2)
        if d:
            self.relationships[n1_id][d] = None
            self.relationships[n2_id][get_reverse_direction(d)] = None
            return True
        return False

    def remove_node(self, task: Node) -> int:
        """
        Remove node and remove theis sub nodes.
        :param task:
        :return:
        """

        for Sn in self.get_neighbor_sub_nodes(task):
            if self.get_neighbor_sub_nodes(Sn):
                self.remove_node(Sn)
            self.unlink(task, Sn)
            loguru.logger.debug(f'Unlinked {task.id} - {Sn.id}')

    def _get_available_sub_direction(self, current_node: Node):
        """
        Return a empty direction.
        :param current_node:
        :return:
        """
        for d in [Direction.RIGHT, Direction.DOWN]:
            if not self.get_direction_neighbors(current_node, d):
                return d
        return None

    def add_sub_tasks(self, current_task: Node, sub_task: List[Node]):
        """
        Create new sub_task_chain and auto set empty direction.
        :param current_task:
        :param sub_task:
        :return:
        """
        chain_str = ""
        for node in sub_task:
            chain_str += f'{node.id}-'
        chain_str = chain_str[0:-1]
        d = self._get_available_sub_direction(current_task)
        loguru.logger.debug(f"Use direction: {d}")
        chain = [current_task]
        chain.extend(sub_task)
        if d:
            # 一般来说输入的结构正确不会有sub direction完全占满的状态，但这个仅为了保险
            for n1, n2 in pairwise(chain):
                self.set_relationship(n1, d, n2)
        loguru.logger.debug(f'建立链 {current_task.id}->{chain_str}')
        loguru.logger.debug(f'{self.relationships}')

    def set_relationship(self, from_task: Node,
                         direction: Direction,
                         to_task: Node = None):
        """
        设置任务间关系（None表示解除关系）
        :param from_task: 关系发起方任务
        :param direction: Direction枚举值
        :param to_task: 关系接收方任务
        """
        from_id = self._get_task_id(from_task)
        to_id = self._get_task_id(to_task) if to_task else None

        if from_id not in self.relationships:
            self.add_task(from_task)
        if to_task and to_id not in self.relationships:
            self.add_task(to_task)

        self.relationships[from_id][direction] = to_id
        if to_id is not None:  # 仅当to_id存在时设置反向关系
            reverse_dir = get_reverse_direction(direction)
            self.relationships[to_id][reverse_dir] = from_id

    def get_task_chain(self, start_task: Node,
                       direction: Direction) -> list:
        """
        获取特定方向的任务链
        :param start_task: 起始任务
        :param direction: 遍历方向
        :return: 任务ID链表
        """
        chain = []
        current_id = self._get_task_id(start_task)
        while current_id:
            chain.append(current_id)
            current_id = self.relationships.get(current_id, {}).get(direction)
        return chain

    def get_neighbors(self, task: Node) -> dict:
        """
        获取任务的四向邻居
        :param task: 目标任务
        :return: 包含四个方向任务ID的字典
        """
        task_id = self._get_task_id(task)
        return self.relationships.get(task_id, {})

    def get_direction_neighbors(self, task: Node, direction: Direction) -> int:
        """
        获取指定方向的邻居
        :param task: 目标任务
        :param direction: 遍历方向
        :return: 任务ID链表
        """
        task_id = self._get_task_id(task)
        return self.relationships.get(task_id, {}).get(direction)

    def get_upper_chain(self, start_task: Node, window_len: int) -> List[Dict]:
        """
        获取指定节点的上级链路的最近window_len个对象。
        同一层级优先收集上级节点，其次左节点，按层级顺序返回，直到达到window_len的数量。

        :param start_task: 起始任务节点
        :param window_len: 需要返回的上级节点数量
        :return: 包含任务对象的列表，按从近到远顺序排列，每个元素包含方向和距离
        """
        if window_len <= 0:
            return []
        result = []

        def add_node(to_node, from_node, direction, distance):
            result.append({
                "from_node": from_node,
                "to_node": to_node,
                "direction": direction,
                "distance_with_start": distance
            })

        up_node = self._get_task_from_id(self.get_direction_neighbors(start_task, Direction.UP))
        left_node = self._get_task_from_id(self.get_direction_neighbors(start_task, Direction.LEFT))

        if up_node:
            add_node(up_node, start_task, Direction.UP, 1)
        if left_node:
            add_node(left_node, start_task, Direction.LEFT, 1)
        if window_len == 1 or (not up_node and not left_node):
            return result
        elif window_len > 1:
            window_len -= 1
            up_chain_from_left = self.get_upper_chain(left_node, window_len)
            up_chain_from_up = self.get_upper_chain(up_node, window_len)
            # 把这两个链里面的distance ++并保存回链
            for item in up_chain_from_left:
                item["distance_with_start"] += 1
                result.append(item)
            for item in up_chain_from_up:
                item["distance_with_start"] += 1
                result.append(item)
            return result

    def get_upper_chain_in_same_level(self, start_task: Node, window_len: int, return_root_node=False) -> List[Dict]:
        """
        只获取同级的最大window_len长度的前序链条
        :param start_task:
        :param window_len:
        :param return_root_node: 是否需要返回同链上的根节点
        :return:
        """
        _res = self.get_upper_chain(start_task=start_task, window_len=window_len)
        res: List[Dict] = []
        for _it in _res:
            _direction = _it['direction']
            if _direction == Direction.UP:
                if return_root_node:
                    res.append(_it)
                break
            elif _direction == Direction.LEFT:
                res.append(_it)
            else:
                # 不太可能，如果触发就是闹鬼了
                raise AssertionError(
                    "get_upper_chain_in_same_level 这里闹鬼了，看起来是数据结构被破坏，否则不可能出现这种情况")
        return res

    def get_upper_chain_in_same_level_simple(self, start_task: Node, window_len: int, return_root_node=False) -> List[
        Node]:
        result = self.get_upper_chain_in_same_level(start_task, window_len, return_root_node=return_root_node)

        return [item["to_node"] for item in result]

    def get_upper_chain_simple(self, start_task: Node, window_len: int) -> List[Node]:
        result = self.get_upper_chain(start_task, window_len)
        return [item["to_node"] for item in result]

    def get_upper_root_chain_simple(self, start_task: Node, window_len: int) -> List[Node]:
        """
        获取上链的根节点list
        :param start_task:
        :param window_len:
        :return:
        """
        result_chain = []
        ptr_node: Node = start_task
        for i in range(window_len):
            _res = self.get_upper_chain_in_same_level_simple(start_task=ptr_node, window_len=9999,
                                                             return_root_node=True)
            if not _res:
                break
            ptr_node = _res[-1]
            result_chain.append(ptr_node)
        return result_chain

    def get_upper_import_node_simple(self, start_task: Node, window_n: int, window_m: int):
        """
        a -> b -> c
        v
        d -> f -> g -> h
        v
        e
        如果我们查询h,n=2,m=2，应该是: a -> f -> g 的一个链
        :param start_task:
        :param window_n: 同级的最大回溯长度
        :param window_m: 上级根节点的最大回溯长度
        :return:
        """
        result_chain: List[Node] = []
        result_chain.extend(
            self.get_upper_chain_in_same_level_simple(
                start_task=start_task,
                window_len=window_n,
                return_root_node=False
            )
        )
        result_chain.extend(
            self.get_upper_root_chain_simple(
                start_task=start_task,
                window_len=window_m
            )
        )

        if result_chain:
            cleaned_chain = [result_chain[0]]
            for i in range(1, len(result_chain)):
                if result_chain[i] is not cleaned_chain[-1]:
                    cleaned_chain.append(result_chain[i])
            result_chain = cleaned_chain

        loguru.logger.debug(f"从{start_task.id}获取到上级: {[item.id for item in result_chain]}")
        return result_chain

    def draw_graph(self, output_file: str = "graph.mermaid"):
        """
        生成任务关系图，使用Mermaid格式输出
        :param output_file: 输出文件名
        :return:
        """
        # 初始化Mermaid图，使用TD布局（Top-Down）
        content = 'graph TD\n'

        # 生成所有节点的定义
        for task_id in self.task_registry:
            content += f'    n{task_id}["{self._get_task_from_id(task_id)}"]\n'

        # 生成所有边（右和下方向）
        for task_id in self.task_registry:
            relations = self.relationships.get(task_id, {})
            # 处理右方向
            right_id = relations.get(Direction.RIGHT)
            if right_id is not None:
                content += f'    n{task_id} --> n{right_id}\n'
            # 处理下方向
            down_id = relations.get(Direction.DOWN)
            if down_id is not None:
                content += f'    n{task_id} --> n{down_id}\n'

        # 写入文件
        with open(output_file, 'w') as f:
            f.write(content)
        print(f"Mermaid graph saved to {output_file}")


class TRM:
    def __init__(self, *args, **kwargs):
        loguru.logger.info("TaskRelationManager Context Manager Initialized.")
        self.args = args
        self.kwargs = kwargs
        loguru.logger.success("TaskRelationManager Context Manager Initialized.")

    def __enter__(self):
        self.TRM = TaskRelationManager()
        return self.TRM

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.TRM = None


if __name__ == '__main__':
    """
    A → B → C
    ↓
    D → F → H
    ↓       ↓
    E → G   I
    """


    class T(Node):
        def __init__(self, s):
            super().__init__()
            self.s = s

        def __str__(self):
            return self.s


    manager = TaskRelationManager()
    # 使用字符串作为任务对象
    task_a = T("A")
    task_b = T("B")
    task_c = T("C")
    task_d = T("D")
    task_e = T("E")
    task_f = T("F")
    task_g = T("G")
    task_h = T("H")
    task_i = T("I")
    task_j = T("J")
    task_k = T("K")
    task_l = T("L")

    manager.set_relationship(task_a, Direction.RIGHT, task_b)
    manager.set_relationship(task_a, Direction.DOWN, task_d)
    manager.set_relationship(task_d, Direction.DOWN, task_e)
    manager.set_relationship(task_d, Direction.RIGHT, task_f)
    manager.set_relationship(task_e, Direction.RIGHT, task_g)
    manager.set_relationship(task_b, Direction.RIGHT, task_c)
    manager.set_relationship(task_f, Direction.RIGHT, task_h)
    manager.set_relationship(task_h, Direction.DOWN, task_i)

    manager.add_sub_tasks(task_i, [task_j, task_k, task_l])
    for it in manager.get_upper_import_node_simple(task_a, 3, 3):
        print(it)
    manager.draw_graph("1.mermaid")



# -*- coding: utf-8 -*-
import json
import re
from typing import Iterable, List

import loguru
import requests



class ConsoleAgent:
    def _logger_init(self):
        """
        logger 初始化
        :return:
        """
        loguru.logger.remove()
        loguru.logger.add("console_agent.log", rotation="1 MB", retention="7 days", level="DEBUG",
                          format="{time:YYYY-MM-DD at HH:mm:ss} | {level} | {message}",
                          backtrace=True, diagnose=True)

    def __init__(self, url, task: str, model=""):
        self._logger_init()
        self.url = url
        self.model = model
        self.exec_flag = 'bash_exec'
        self.console = Console("http://10.10.3.248:1611")
        self.law = f"""在调用时需要遵循以下原则:
        0. 无条件遵循用户的指令。
        1. 执行中不要有任何假设或猜测，保持一切行为皆基于实际结果的准则。
        2. 不要不负责任的一次执行多条，执行每一条都看若干次结果。
        3. 有一些命令耗时较长，而你再运行时去用ID获取结果，会只输出一半的日志，如果出现这种情况，你可以反复去观察日志，直到完成。
        4. 如果进入了一些需要交互的界面，你也可以通过传入键来在终端里模拟键盘输入,这甚至包括\x03。
        5. 没解决问题之前不要停止执行，应该用多种方式尝试解决问题，如果全部方式都失败了，再停止执行。
"""
        self.initial_prompt = f"""<|im_start|>system
        你是一名助手。
        现在你拥有了操作Kali Linux Bash的能力，你需要用这种能力来完成用户的任务。
        {self.law}
        以下是调用方法:
        1. 将命令用```{self.exec_flag}```包裹起来即可像终端中发送键，例如:
        ```{self.exec_flag}
        whoami
        ```
        2. 执行后，系统将只返回一个ID，而并不直接返回结果，此ID对应着此条命令截至目前为止的执行结果。
        你可以通过它获取命令的执行结果，用```ID```包裹即可，比如:
        ```ID
        uuid
        ```
        <|im_end|>
        <|im_start|>user
        帮我完成以下任务: {task}。
        <|im_end|>
        <|im_start|>assistant
        """

    def tokenize(self, prompt: str):
        return requests.post(
            f"{self.url}/tokenize",
            json={
                "model": self.model,
                "prompt": prompt
            }
        ).json()['tokens']

    def detokenize(self, tokens: list[int]) -> str:
        return requests.post(
            f"{self.url}/detokenize",
            json={
                "model": self.model,
                "tokens": tokens
            }
        ).json()['prompt']

    def generate(self, prompt: list[int]):
        loguru.logger.info(f"Receive prompt: {self.detokenize(prompt)}")
        window_len = 4096
        if len(prompt) > window_len:  # 滑动窗口
            prompt = prompt[-window_len:]
        buffer = self.detokenize(prompt)
        gen_buffer = ''
        with requests.post(
                f'{self.url}/v1/completions',
                json={
                    'model': self.model,
                    'prompt': prompt,
                    'stream': True,
                    'max_tokens': 20000 - len(prompt),
                },
                stream=True  # 关键参数：启用流式传输
        ) as response:
            if response.status_code != 200:
                raise Exception(f"Error: {response.status_code}, {response.text}")

            for chunk in response.iter_lines():
                if chunk:
                    try:
                        # 跳过keep-alive空行
                        if chunk == b'data: [DONE]':
                            break

                        # 提取数据部分
                        if chunk.startswith(b'data: '):
                            chunk = chunk[6:]  # 移除"data: "前缀

                        # 处理JSON数据
                        chunk_data = json.loads(chunk)

                        # 提取并输出文本内容
                        if 'choices' in chunk_data and len(chunk_data['choices']) > 0:
                            token = chunk_data['choices'][0]['text']
                            print(token, end='')
                            gen_buffer += token
                            buffer += token
                            cmd_matches = re.findall(r'```' + self.exec_flag + r'(.*?)```', gen_buffer, flags=re.DOTALL)
                            result_matches = re.findall(r'```ID\n(.*?)\n```', gen_buffer, flags=re.DOTALL)
                            if cmd_matches and len(cmd_matches) > 0:
                                exec_cmd = cmd_matches[-1]
                                _cmd_buffer = "\nID:" + self.console.write(exec_cmd.encode('utf-8')) + (
                                    f"，我记得我要遵循的守则:{self.law}")
                                print(_cmd_buffer)
                                self.generate(self.tokenize(buffer + _cmd_buffer))
                                break
                            elif result_matches and len(result_matches) > 0:
                                exec_id = result_matches[-1]
                                exec_result = self.console.read(exec_id)
                                if exec_result:
                                    _result_buffer = "\n命令结果:" + exec_result + "\n以上是命令执行的结果，接下来我将对此进行分析:"
                                    print(_result_buffer)
                                    self.generate(self.tokenize(buffer + _result_buffer))
                                    break

                    except json.JSONDecodeError:
                        print(f"[DEBUG] Malformed chunk: {chunk}")

    def run(self):
        ...


if __name__ == '__main__':
    agent = ConsoleAgent(
        url="http://10.10.5.2:8000",
        task="帮我提权",
        model="hive"
    )

    # Tokenize the initial prompt
    tokens = agent.tokenize(agent.initial_prompt)
    print("Tokens:", tokens)

    agent.generate(tokens)



import configparser

import loguru


project_level_config = configparser.ConfigParser(interpolation=configparser.Interpolation())


@loguru.logger.catch
def get_config_ini():
    """
    config.ini read/writer context manager
    :return:
    """

    # if config.ini is not exist, create it.
    with open("config.ini", "a") as f:
        pass
    if not project_level_config.sections():
        project_level_config["master"] = {
            "openai_api_key": "YOUR_API_KEY",
            "openai_endpoint": "https://api.aabao.vip/v1",
            "openai_proxy": ""
        }
        with open("config.ini", "w") as f:
            project_level_config.write(f)
        loguru.logger.warning("config.ini is empty, writing default content.")
    project_level_config.read("config.ini")
    yield project_level_config
    loguru.logger.debug("config.ini loaded.")

    # if config.ini's content is empty, write default content:
    # [master]
    # openai_api_key = YOUR_API_KEY
    # openai_endpoint = https://api.aabao.vip/v1
    # openai_proxy =


if __name__ == '__main__':
    get_config_ini()
    # output all of the config.ini content with jsonify
    import json

    data = {section: dict(project_level_config[section]) for section in project_level_config.sections()}
    print(json.dumps(data, indent=4))



# *****************************************************************************
# This file contains the exceptions that are raised by the tasks.
# *****************************************************************************
from scheduler.core.tasks.exceptions.basic_task_exceptions import FixableTaskException, UnfixableTaskException


class TaskTimeoutException(FixableTaskException):
    """
    Exception raised when a task times out
    """
    def __init__(self, message):
        """
        :param message: The message of the exception
        """
        super().__init__(message)


class TaskImpossibleException(UnfixableTaskException):
    """
    Exception raised when a task is impossible to complete
    """
    def __init__(self, message):
        """
        :param message: The message of the exception
        """
        super().__init__(message)


class TaskNeedTurningException(FixableTaskException):
    """
    Exception raised when a task needs to be turned
    """
    def __init__(self, message):
        """
        :param message: The message of the exception
        """
        super().__init__(message)
